package animation.balg;

import static java.lang.Math.PI;
import static java.lang.Math.atan2;
import static java.lang.Math.cos;
import static java.lang.Math.pow;
import static java.lang.Math.sin;
import static java.lang.Math.toDegrees;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import animation.pendel.PanelPendel;

public class PanelBalg extends PanelPendel {

	private static final long serialVersionUID = 1L;

	private final int LEER = 0;
	private final int LEEREND = 1;
	private final int FUELLEND = 2;
	private final int VOLL = 3;

	final int ANZAHL_SEKTOREN = 12;

	final double KREIS_RADIUS = 140;

	final double MASSE = 6.0;
	final double SKALIERUNG = 0.005; // 1 Pixel = 5mm

	private final double LEER_WINKEL = 0.01;
	private final double VOLL_WINKEL = 360.0 / ANZAHL_SEKTOREN;

	private boolean dgl_init_dynamisch = false;
	private PanelBalgParameter parameterPanel;
	private int versatz;

	private RadDaten balgDaten;

	private double phi_alt;
	private double winkel_summe = 0;

	// double r;
	// double masse;

	// double g;
	// double phi_mechanik;
	// double dw;

	ArrayList<Balg> balge = new ArrayList<Balg>();
	Rectangle2D.Double rolle;
	Line2D.Double rampe;

	public PanelBalg(PanelBalgParameter parameterPanel) {
		this.parameterPanel = parameterPanel;

		balgDaten = new RadDaten();

		initDgl();

		// Segmentlaenge bestimmen, das ist gleichzeitig der Balgradius.
		// Gemeint sind die beiden Schnittpunkte des Segments mit dem Kreis.
		// Dieser Wert ist immer gleich.
		double balgRadius = 2 * KREIS_RADIUS * sin(PI / ANZAHL_SEKTOREN);
		versatz = (ANZAHL_SEKTOREN / 2) + 1;
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			Balg balg = new Balg(i);
			balg.setRadius(balgRadius);
			balge.add(balg);
		}

		// Die "Rolle", die den Balg unten schliesst
		double xrect = WIDTH / 2 - 2;
		double yrect = HEIGHT / 2 + KREIS_RADIUS + 1;
		rolle = new Rectangle.Double(xrect, yrect, 4.0, 3.0);
		// rolle = new Arc2D.Double(xrect,yrect,20,20,0,360,Arc2D.PIE);
		// rampe = new Line2D.Double(xrect-20,yrect+20, xrect,yrect);

		// Kreiszentrum
		balgDaten.setXz(WIDTH / 2);
		balgDaten.setYz(HEIGHT / 2);
		balgDaten.setSkalierung(SKALIERUNG);
		balgDaten.setMasse(MASSE);
		balgDaten.setGravitationskonstante(9.81);
	}

	/*
	private boolean intersectLines(Line2D l, Line2D m)
			throws IllegalArgumentException {
		// Quelle:
		// http://forum.worldofplayers.de/forum/threads/783387-JAVA-Methode-zum-Schnittpunkt-zweier-Geraden(am-besten-Line2D-Objecte)-berechnen

		// Wegen der Lesbarkeit
		double lx1 = l.getX1();
		double lx2 = l.getX2();
		double ly1 = l.getY1();
		double ly2 = l.getY2();

		double mx1 = m.getX1();
		double mx2 = m.getX2();
		double my1 = m.getY1();
		double my2 = m.getY2();

		if ((lx1 == mx1) && (lx2 == mx2) && (ly1 == my1) && (ly2 == my2)) {
			System.out.println("Gleich.");
			return true;
		}

		// Zaehler
		double zx = (lx1 * ly2 - ly1 * lx2) * (mx1 - mx2) - (lx1 - lx2)
				* (mx1 * my2 - my1 * mx2);
		double zy = (lx1 * ly2 - ly1 * lx2) * (my1 - my2) - (ly1 - ly2)
				* (mx1 * my2 - my1 * mx2);

		// Nenner
		double n = (lx1 - lx2) * (my1 - my2) - (ly1 - ly2) * (mx1 - mx2);

		// Koordinaten des Schnittpunktes
		double x = zx / n;
		double y = zy / n;

		// Vielleicht ist bei der Division durch n etwas schief gelaufen
		if (Double.isNaN(x) & Double.isNaN(y)) {
			System.out.println("Schnittpunkt nicht eindeutig.");
			return false;
		}
		// Test ob der Schnittpunkt auf den angebenen Strecken liegt oder
		// auerhalb.
		if ((x - lx1) / (lx2 - lx1) > 1 || (x - mx1) / (mx2 - mx1) > 1
				|| (y - ly1) / (ly2 - ly1) > 1 || (y - my1) / (my2 - my1) > 1) {
			System.out.println("Schnittpunkt liegt auerhalb.");
			return false;
		}
		System.out.println("Schnittpunkt: " + x + "," + y);
		return true;
	}
	*/

	private void balgeZeichnen(Graphics2D g2, double phi, ArrayList<Balg> balge) {

		double alpha0;
		double alpha1;
		double beta;

		double x0, y0, x1, y1;
		double x0b, y0b, x1b, y1b;

		double winkel_leerend = 0;
		double winkel_fuellend = 0;
		double winkel_rest = 0;

		double winkel_alt = 0;
		double winkel_differenz = 0;

		int k, n;
		Arc2D.Double balg;

		// Rad um 1 Winkelschritt drehen und neue Koordinaten der Balge
		// bestimmen
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			// Rad rotiert im Gegenuhrzeigersinn

			// Karthesisches Koordinatesystem:
			// Schnittpunkte der Balg-Segmente mit dem Rad bestimmen.
			// 
			// Kreiszentrum = P(0,0)
			alpha0 = phi + ((PI * 2.0 * (double) i) / (double) ANZAHL_SEKTOREN);
			alpha1 = phi
					+ ((PI * 2.0 * (double) (i + 1)) / (double) ANZAHL_SEKTOREN);

			x0 = cos(alpha0) * KREIS_RADIUS;
			y0 = sin(alpha0) * KREIS_RADIUS;
			x1 = cos(alpha1) * KREIS_RADIUS;
			y1 = sin(alpha1) * KREIS_RADIUS;

			// Bildschirm-Koordinaten:
			// P(0,0) = oben links, P(maxX,maxY) = unten rechts

			// Balg- bzw. Traeger-Koordinaten bezogen auf Bildschirmkoordinaten
			x0b = x0 + WIDTH / 2;
			y0b = HEIGHT / 2 - y0;
			x1b = x1 + WIDTH / 2;
			y1b = HEIGHT / 2 - y1;

			beta = toDegrees(atan2((y0 - y1), (x0 - x1)));
			beta -= 360 / ANZAHL_SEKTOREN;

			// Alle Balge auf VOLL setzen, spaeter modifizieren.
			balge.get(i).setArcByCenter(x0b, y0b, beta, VOLL_WINKEL, Arc2D.PIE);
			balge.get(i).getTraeger().x1 = x0b;
			balge.get(i).getTraeger().x2 = x1b;
			balge.get(i).getTraeger().y1 = y0b;
			balge.get(i).getTraeger().y2 = y1b;
		}

		// Balge modifizieren und ihren Status richtig setzen.
		n = -1;
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			// Den zu leerenden Balg suchen, das ist der, der die Rolle
			// unten beruehrt.
			balg = balge.get(i).getArc();
			//Point2D.Double sp = new Point2D.Double(balge.get(i).getTraeger().x1,balge.get(i).getTraeger().y1);
			
			winkel_leerend = VOLL_WINKEL;

			
			//Line2D.Double line = new Line2D.Double(sp,balg.getEndPoint());
			//g2.draw(line);
			//intersectLines(line, Line2D m)
			while (balg.intersects(rolle)) {
				// Den Balg, der die Rolle beruehrt, in 1/100 Grad
				// so lange weiter schliessen, bis er die Rolle nicht mehr
				// berhrt.
				// Das ist eine Vereinfachung.
				winkel_leerend -= 0.01;
				balg.setAngleExtent(winkel_leerend);
				n = i; // Balgnummer speichern
			}
			if (n == i) { // Der Balg wurde gefunden
				break;
			}
		}

		if (n < 0) {
			g2.drawString("Rolle nicht gefunden", WIDTH / 2 - 50, HEIGHT / 2);
			return;
		}

		// Wenn es einen Balg gibt, der die Rolle beruehrt, so ist n
		// der Index auf diesen Balg.
		// Von n aus die Balge im Gegenuhrzeigersinn modifizieren.
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			k = (n + i) % ANZAHL_SEKTOREN;
			if (i == 0) {
				// k zeigt auf den Balg, der geleert wird

				// Wurde im Schritt vorher der vorherige Balg geleert?
				winkel_rest = 0.0;
				for (int m = 0; m < ANZAHL_SEKTOREN; m++) {
					if (balge.get(m).getStatus() == LEEREND) {
						if (m != k) { // Ja. Der Balg vorher war leerend.

							// Die verbliebene Wassermenge muss noch verrechnet
							// werden
							winkel_rest = balge.get(m).getOeffnungsWinkel()
									- LEER_WINKEL;
							winkel_summe = 0.0;
						}
						break;
					}
				}
				// Balg, der aktuell geleert wird;
				balge.get(k).setStatus(LEEREND);
				winkel_alt = balge.get(k).getOeffnungsWinkel();
				winkel_differenz = winkel_alt - winkel_leerend + winkel_rest;
				winkel_summe = winkel_summe + winkel_differenz;
				balge.get(k).setOeffnungsWinkel(winkel_leerend);

			} else if (i < versatz) {
				// Die im Gegenuhrzeigersinn liegenden Balge auf leer setzen.
				balge.get(k).setStatus(LEER);
				balge.get(k).setOeffnungsWinkel(LEER_WINKEL);
			} else if (i == versatz) {
				// Den oberen Balg fuellen.
				balge.get(k).setStatus(FUELLEND);
				winkel_fuellend = VOLL_WINKEL - winkel_leerend;
				balge.get(k).setOeffnungsWinkel(winkel_fuellend);
			} else {
				// Die restlichen Balge sind voll
				balge.get(k).setStatus(VOLL);
				balge.get(k).setOeffnungsWinkel(VOLL_WINKEL);
			}
		}

		// Die Balge zeichnen und entsprechend ihrem Oeffnungswinkel einfaerben
		g2.setColor(Color.blue);
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			g2.fill(balge.get(i).getArc());
		}

		// Die Traeger zeichnen und einfaerben
		g2.setColor(Color.black);
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			g2.draw(balge.get(i).getTraeger());
		}
	}

	private void zeichneSchwerpunkt(Graphics2D g2, Color color,
			RadDaten balgDaten) {
		g2.setColor(color);
		double xs = balgDaten.getXsPixel();
		double ys = balgDaten.getYsPixel();
		g2.draw(new Line2D.Double(xs - 3, ys, xs + 3, ys));
		g2.draw(new Line2D.Double(xs, ys - 3, xs, ys + 3));
	}

	private void zeichneZentrum(Graphics2D g2, Color color, RadDaten balgDaten) {
		g2.setColor(color);
		double xz = balgDaten.getXz();
		double yz = balgDaten.getYz();
		g2.draw(new Line2D.Double(xz - 3, yz, xz + 3, yz));
		g2.draw(new Line2D.Double(xz, yz - 3, xz, yz + 3));
	}

	/*
	 * protected void getMomentOfInertia(int width, int height,double xz, double
	 * yz, Shape shape,double scaling, double mass) { double I = 0.0; int area =
	 * 0; for (int x=0; x<width; x++) { for (int y=0; y<height; y++) { if
	 * (shape.contains(x, y)) { I += pow((x-xz)*scaling, 2)+pow((yz-y)*scaling,
	 * 2); area += 1; } } } balgDaten.setMassentraegheit(I * mass /area); }
	 */

	private void balgeRechnen(Graphics2D g2, RadDaten balgDaten,
			ArrayList<Balg> balge) {
		int x0, x1, y0, y1;
		double xs = 0;
		double ys = 0;
		double area = 0;

		double xs_total = 0;
		double ys_total = 0;
		double area_total = 0;
		double I = 0.0;

		// Zentrum
		double xz = balgDaten.getXz();
		double yz = balgDaten.getYz();

		double skalierung = balgDaten.getSkalierung();
		double masse = balgDaten.getMasse();

		Balg balg;
		Arc2D.Double arc;

		g2.setColor(Color.yellow);

		// System.out.println("Neuer Schritt: ");

		xs_total = 0;
		ys_total = 0;
		area_total = 0;
		for (int i = 0; i < ANZAHL_SEKTOREN; i++) {
			balg = balge.get(i);
			/*
			 * switch(balg.getStatus()) { case VOLL: System.out.print("Voll ");
			 * break; case FUELLEND: System.out.print("Fuellend "); break; case
			 * LEER: System.out.print("Leer "); break; case LEEREND:
			 * System.out.print("Leerend "); break; } System.out.println("i:
			 * "+i+" A: "+balg.getArea()+" da: "+balg.getDeltaArea());
			 */
			if (balg.getStatus() != LEER) {
				// Shape shape = balg.arc;
				arc = balg.getArc();
				x0 = ((int) arc.getMinX()) - 2;
				x1 = ((int) arc.getMaxX()) + 2;
				y0 = ((int) arc.getMinY()) - 2;
				y1 = ((int) arc.getMaxY()) + 2;
				xs = 0;
				ys = 0;
				area = 0;
				// Flaechen und Traegheitsmoment graphisch bestimmen.
				for (int x = x0; x < x1; x++) {
					for (int y = y0; y < y1; y++) {
						if (arc.contains(x, y)) {
							xs += x;
							ys += y;
							xs_total += x;
							ys_total += y;
							// Pixelkoordinatensystem: Ppixel(0,0) = Obere
							// Bildschirm-Ecke links
							// Zentrum Ppixel(xz,yz) = Bildschirmmitte
							//
							// Karthesisches Koordinatensystem: Pkarth(0,0) =
							// Bildschirm-Mitte
							// Daher x-xz und yz-y statt x-zz und y-yz
							I += pow((x - xz) * skalierung, 2)
									+ pow((yz - y) * skalierung, 2);
							area += 1;
							area_total += 1;
						}
					}
				}
				if (area > 0) {
					xs /= area;
					ys /= area;

					balg.setXs(xs);
					balg.setYs(ys);

					// Scherpunkt des Balges eintragen
					g2.draw(new Line2D.Double(xs - 3, ys, xs + 3, ys));
					g2.draw(new Line2D.Double(xs, ys - 3, xs, ys + 3));
				}
			}
		}
		xs_total /= area_total;
		ys_total /= area_total;

		balgDaten.setXsPixel(xs_total);
		balgDaten.setYsPixel(ys_total);
		balgDaten.setAreaPixel(area_total);
		balgDaten.setMassentraegheitsmoment(I * masse / area_total);
	}

	public void reset() {
		// Der Aufrufer von reset() ruft am Schluss repaint() auf.
		// Damit zeichnet repaint() zuerst die Grafik mit phi = 0.0.
		// Anschliessend wird in repaint() getestet, ob die Dgl()-Werte
		// schon initalisiert sind.
		// Wegen dgl_init_dynamisch==false erfolgt in diesem repaint()
		// das Initialisieren der Dgl():
		// phi = -168.0;
		dgl_init_dynamisch = false;
	}

	private void updateRad(Graphics2D bg2, double phi, RadDaten balgDaten,
			ArrayList<Balg> balge) {
		// Balge zeichnen
		bg2.setColor(Color.blue);
		balgeZeichnen(bg2, phi, balge);

		// Flaechentraegheitsmoment und Schwerpunkte rechnen
		balgeRechnen(bg2, balgDaten, balge);

		// Schwerpunkte zeichnen
		zeichneSchwerpunkt(bg2, Color.black, balgDaten);
		zeichneZentrum(bg2, Color.black, balgDaten);

		// Drehrichtung zeichnen
		if (phi >= phi_alt) {
			bg2.drawString("v", 350, 350);
		} else {
			bg2.drawString("r", 350, 350);
		}
		phi_alt = phi;

		// Rolle zeichnen
		// Rolle zeichnen
		bg2.setColor(Color.black);
		bg2.fill(rolle);
	}

	protected void paintComponent(Graphics gr) {

		Graphics2D g2 = (Graphics2D) gr;

		// Zuerst in ein BufferedImage zeichnen.
		// Ueber dieses kann ggf. eine GIF Animation erstellt werden.
		BufferedImage img = new BufferedImage(WIDTH, HEIGHT,
				BufferedImage.TYPE_INT_RGB);
		Graphics bg = img.createGraphics();
		Graphics2D bg2 = (Graphics2D) bg;

		bg2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		// Hintergrund zeichnen
		bg2.clearRect(0, 0, WIDTH, HEIGHT);
		bg2.setColor(Color.WHITE);
		bg2.fillRect(0, 0, WIDTH, HEIGHT);

		if (dgl_init_dynamisch == false) {
			dgl_init_dynamisch = true;
			phi = Math.toRadians(-180.0);
			updateRad(bg2, phi, balgDaten, balge);
			// phi = balgDaten.getPhi();
			t = 0.0;
			w = 0.0;
			L = balgDaten.getSchwerpunktradius();
			M = balgDaten.getMasse();
			I = balgDaten.getMassenTraegheitsmoment();
			g = balgDaten.getGravitationskonstante();
			dw = 8;// -(M*g*L*sin(balgDaten.getPhi())/I)*dt;
			g2.drawImage(img, 0, 0, null);
			// Dgl();
		} else {

			updateRad(bg2, phi, balgDaten, balge);

			Dgl();
			// Ggf. neues Image ins animierte GIF eintragen
			parameterPanel.writeImage(img);

			parameterPanel.addRow(t, phi, w, balgDaten.getAreaQuadratmeter(),
					balgDaten.getXsMeter(), balgDaten.getYsMeter(), balgDaten
							.getMassenTraegheitsmoment());

			// Image auf dem Bildschirm darstellen
			g2.drawImage(img, 0, 0, null);

		}

	}

	protected void Dgl() {

		L = balgDaten.getSchwerpunktradius();
		M = balgDaten.getMasse();
		I = balgDaten.getMassenTraegheitsmoment();
		g = balgDaten.getGravitationskonstante();
		double phi_mechanik = balgDaten.getPhi();

		// dw/dt = Aenderung der Winkelgeschwindigkeit

		dw = -(M * g * L * sin(phi_mechanik) / I) * dt;

		w = w + dw; // w= Winkelgeschwindigkeit = dphi/dt

		phi += w * dt;

		t = t + dt;

		/*
		 * double r = balgDaten.getSchwerpunktradius(); double masse =
		 * balgDaten.getMasse(); double I =
		 * balgDaten.getMassenTraegheitsmoment(); double g =
		 * balgDaten.getGravitationskonstante();
		 * 
		 * 
		 * phi = balgDaten.getPhi();
		 * 
		 * //double dphi = phi_alt - phi; //phi_alt = phi; //phi += dphi;
		 * //double dw = (phi_alt - phi) / dt;
		 * 
		 * 
		 * double dw = (-masse*g*r*sin(phi)/I)*dt;
		 * 
		 * w = w + dw; // w= Winkelgeschwindigkeit = dphi/dt
		 * 
		 * //phi = w * dt; if (phi > (2 * PI)) { phi = phi - (2*PI); }
		 * 
		 * t = t + dt;
		 */
	};

	protected void initDgl() {
		phi = 0.0;
		phi_alt = 0.0;
		w = 0.0;
		t = 0.0;
		dt = 0.01;
		dgl_init_dynamisch = false;
	};

	@Override
	protected void clearTable() {
		parameterPanel.clearTable();

	}

}