package animation.balg;

import gui.DecimalField;
import gui.ImageFilter;
import imaging.GifSequenceWriter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.stream.FileImageOutputStream;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import animation.pendel.PanelParameter;


public class PanelBalgParameter extends PanelParameter {

	private static final long serialVersionUID = 1L;
	private final int TABLE_WIDTH = 400;
	private final int TABLE_HEIGHT = 293;
	
	private JTable table;

	private DecimalField dfMasse;
	private DecimalField dfLaenge;
	private DecimalField dfPhi;
	
	private JButton butCreateGIF;
	private boolean bGifErzeugen;
	private GifSequenceWriter writer;
	private ImageOutputStream output;

	public PanelBalgParameter() {

		dfMasse = new DecimalField(0, 3);
		dfLaenge = new DecimalField(0, 3);
		dfPhi = new DecimalField(0, 3);
		
		butCreateGIF = new JButton("GIF erzeugen");
		butCreateGIF.setBounds(290,10,110,20);
		add(butCreateGIF);
		butCreateGIF.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0) {
				cmdGifErzeugen();
			}});

		setInitialParameters();
		
		bGifErzeugen = true;

		initTable(this);
	}

	private void setInitialParameters() {
		setParameter(this, dfMasse, "Masse [Kg]", 0, 10, 6.0, 0.1, 60);
		setParameter(this, dfLaenge, "Teilung", 0, 35, 12, 5, 25);
		setParameter(this, dfPhi, "Durchmesser [m]", 0, 60, 1.4, 0.7, 1.4);
	}
	
	public void reset() {
		setInitialParameters();
		
		clearTable();
		
		butCreateGIF.setText("GIF erzeugen");
		if (writer != null) {
			try {
				writer.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (output != null) {
			try {
				output.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		bGifErzeugen = true;
		
	}
	
	public double getG() {
		return 9.81;
	}

	public double getMasse() {
		return dfMasse.getValue();
	}

	public double getLaenge() {
		return dfLaenge.getValue();
	}

	public double getPhi() {
		double phi = dfPhi.getValue();
		return Math.toRadians(phi);
	}

	public void writeImage(BufferedImage img) {
		if (writer == null) return;
		try {
			writer.writeToSequence(img);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void mnu_fileSaveAs() {

		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(new File("GR-Rad-305.gif"));
		ImageFilter imageFilter = new ImageFilter();
		chooser.addChoosableFileFilter(imageFilter);
		chooser.setAcceptAllFileFilterUsed(false);

		int rval = chooser.showSaveDialog(this);
		if (rval == JFileChooser.APPROVE_OPTION) {
			File fileToSave = chooser.getSelectedFile();
			if (imageFilter.accept(fileToSave)) {
				if (fileToSave.exists()) {
					int response = JOptionPane.showConfirmDialog(null,
							"Overwrite existing file?", "Confirm Overwrite",
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (response == JOptionPane.CANCEL_OPTION)
						return;
				}
				
				try {
					
					output = new FileImageOutputStream(fileToSave);
					writer = new GifSequenceWriter(
							output, BufferedImage.TYPE_INT_RGB, 50, true);
					
					
					
				} catch (IOException ex) {
					JOptionPane.showMessageDialog(this, ""
							+ "File cannot be saved", "File Error",
							JOptionPane.ERROR_MESSAGE);
				}
			} else {
				JOptionPane.showMessageDialog(this, ""
						+ "File cannot be saved (wrong extension).",
						"File Error", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public void addRow(double t, double phi, double omega, double area,
			double xs, double ys, double I) {
		Vector<String> row = new Vector<String>();

		row.addElement(String.format("%.3f", t));
		row.addElement(String.format("%.3f", Math.toDegrees(phi)));
		row.addElement(String.format("%.3f", Math.toDegrees(omega)));
		row.addElement(String.format("%.3f", area));
		row.addElement(String.format("%.3f", xs));
		row.addElement(String.format("%.3f", ys));
		row.addElement(String.format("%.3f", I));
		((DefaultTableModel) table.getModel()).addRow(row);

	}

	@Override
	protected void initTable(JPanel p) {
		columnNames = new Vector<String>();
		columnNames.addElement("t [s]");
		columnNames.addElement("Phi []");
		columnNames.addElement("Omega []");
		columnNames.addElement("Area [m^2]");
		columnNames.addElement("Xs [m]");
		columnNames.addElement("Xs [m]");
		columnNames.addElement("I [Kg*m^2]");

		rowData = new Vector<Vector<String>>();
		table = new JTable(rowData, columnNames) {

			private static final long serialVersionUID = 1L;
			DefaultTableCellRenderer renderRight = new DefaultTableCellRenderer();
			{// initializer block
				renderRight.setHorizontalAlignment(SwingConstants.RIGHT);
			}

			@Override
			public TableCellRenderer getCellRenderer(int arg0, int arg1) {
				return renderRight;
			}
		};
		table.getColumnModel().getColumn(0).setPreferredWidth(50);
		table.getColumnModel().getColumn(1).setPreferredWidth(70);
		table.getColumnModel().getColumn(2).setPreferredWidth(80);
		table.getColumnModel().getColumn(3).setPreferredWidth(80);
		table.getColumnModel().getColumn(4).setPreferredWidth(70);
		table.getColumnModel().getColumn(5).setPreferredWidth(70);

		table.setFillsViewportHeight(true);

		JScrollPane scrollPane = new JScrollPane(table);

		scrollPane.setBounds(0, 90, TABLE_WIDTH, TABLE_HEIGHT);
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		p.add(scrollPane);

	}

	public void cmdGifErzeugen() {
		if (bGifErzeugen==true) {
			bGifErzeugen = false;
			butCreateGIF.setText("GIF beenden");
			mnu_fileSaveAs();
		}
		else {
			bGifErzeugen = true;
			butCreateGIF.setText("GIF erzeugen");
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if (output != null) {
				try {
					output.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	public void clearTable() {
		((DefaultTableModel) table.getModel()).setRowCount(0);
	}
}
