package animation.pendel;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import gui.DecimalField;
import gui.ExcelAdapter;
import gui.MainFrame;
import gui.guiConsts;


public class DlgPendelMathematisch extends DlgPendel {

	private static final long serialVersionUID = 1L;
	
	private final int DLG_PANEL_WIDTH = 800;
	private final int DLG_PANEL_HEIGHT = 520;
	
	private final int DRAWPANEL_X = 20;
	private final int DRAWPANEL_Y = 20;
	private final int DRAWPANEL_WIDTH = 440;
	private final int DRAWPANEL_HEIGHT = 440;

	private final int PARAM_PANEL_X = 480;
	private final int PARAM_PANEL_Y = 10;
	private final int PARAM_PANEL_WIDTH = 300;
	private final int PARAM_PANEL_HEIGHT = 400;
	
	private final int BUT_X = PARAM_PANEL_X;
	private final int BUT_Y = PARAM_PANEL_HEIGHT + 35;
	private final int WBUT = 80;
	private final int HBUT = 25;
	
	
	private PanelPendelParameter parameterPanel;
	private PanelPendelMathematisch pendelPanel;
	
	public DlgPendelMathematisch(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
		setTitle("Mathematisches Pendel");
		setSize(DLG_PANEL_WIDTH,DLG_PANEL_HEIGHT);
		
		parameterPanel = new PanelPendelParameter();
		parameterPanel.setBounds(PARAM_PANEL_X, PARAM_PANEL_Y, PARAM_PANEL_WIDTH, PARAM_PANEL_HEIGHT);
		pBackground.add(parameterPanel);
		
		pendelPanel = new PanelPendelMathematisch(parameterPanel);
		pendelPanel.setBounds(DRAWPANEL_X, DRAWPANEL_Y, DRAWPANEL_WIDTH, DRAWPANEL_HEIGHT);
		pBackground.add(pendelPanel);	
		
		JButton butStart = new JButton("Start");
		butStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdStart();
			}});
		
		butStart.setBounds(BUT_X,BUT_Y,WBUT,HBUT);
		pBackground.add(butStart);
		
		JButton butStop = new JButton("Stop");
		butStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdStop();
			}});
		
		butStop.setBounds(BUT_X+WBUT+10,BUT_Y,WBUT,HBUT);
		pBackground.add(butStop);
		
		JButton butCancel = new JButton("Beenden");
		butCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdCancel();
			}});
		butCancel.setBounds(BUT_X+2*WBUT+20,BUT_Y,WBUT+20,HBUT);
		pBackground.add(butCancel);
	}

	private void cmdStart() {
		pendelPanel.startAnimation();
	}
	
	private void cmdStop() {
		pendelPanel.stopAnimation();
		
	}
	
	protected void cmdCancel() {
		pendelPanel.stopAnimation();
		pendelPanel.setVisible(false);
		dispose();
	}

	
	private class PanelPendelMathematisch extends PanelPendel  {

		private static final long serialVersionUID = 1L;

		private PanelPendelParameter parameterPanel;
		
		private PanelPendelMathematisch(PanelPendelParameter parameterPanel) {
			this.parameterPanel = parameterPanel;
			initDgl();
		}
		
		
		protected void initDgl() {
			g = parameterPanel.getG();
			L = parameterPanel.getLaenge();
			phi = parameterPanel.getPhi();
			w = 0.0;
			t = 0.0;
			dt = 0.05; 
		}


		@Override
		protected void Dgl() {
			// Winkel des Fadenpendels (im Bogenmass)
			double dw; // Winkelbeschleunigung
			parameterPanel.addRow(t, phi, w);
			
			//dw = -(g/L)*phi * dt;  // Nherung bei kleinen phi wegen x = sin(x)
			dw = -(g/L)*Math.sin(phi) * dt; // Genau
			w = w + dw;
			phi = phi + w * dt;
			t = t + dt;
		}


		@Override
		protected void clearTable() {
			parameterPanel.clearTable();
		}
		
		protected void paintComponent(Graphics g) {

			
			int x;
			int y;

			int len = (int)(Math.round(L * 100.0));

		
			super.paintComponent(g);
			
			if (offscreen==null) {
				offscreen = createImage(WIDTH, HEIGHT);
				bg = offscreen.getGraphics();
			}
			
			if (bg != null) {
				bg.clearRect(0, 0, WIDTH, HEIGHT);
				bg.setColor(Color.WHITE);
				bg.fillRect(0, 0, guiConsts.imageDrawPanelWidth,
						guiConsts.imageDrawPanelHeight);

				bg.setColor(Color.BLACK);
				y = (int) Math.round(len * Math.cos(phi));
				x = (int) Math.round(len * Math.sin(phi));
				bg.fillOval(WIDTH / 2 - 2, HEIGHT / 2 - 2, 4, 4);
				bg.drawLine(WIDTH / 2, HEIGHT / 2, WIDTH / 2 + x, WIDTH / 2 + y);
				bg.fillOval(WIDTH / 2 + x - 4, HEIGHT / 2 + y - 4, 8, 8);
				Dgl();

				g.drawImage(offscreen, 0, 0, this);
			}
		}

	}

	private class PanelPendelParameter extends PanelParameter {

		private static final long serialVersionUID = 1L;
		
		private DecimalField dfG;
		private DecimalField dfLaenge;
		private DecimalField dfPhi;
		
		private JTable table;
				
		private PanelPendelParameter() {
		
			dfG = new DecimalField(0, 3);
			dfLaenge = new DecimalField(0, 3);
			dfPhi = new DecimalField(0, 3);
				
			setParameter(this, dfG, "g [m/s^2]", 0, 10, 9.81, 0.0, 20.0);
			setParameter(this, dfLaenge, "Laenge [m]", 0, 35, 0.994, 0.1, 2.0);
			setParameter(this, dfPhi, "Phi []", 0, 60, 90, 0, 360);
			
			initTable(this);
		}
		

		protected void initTable(JPanel p) {

			columnNames = new Vector<String>();
			columnNames.addElement("t");
			columnNames.addElement("Phi");
			columnNames.addElement("Omega");

			rowData = new Vector<Vector<String>>();
			table = new JTable(rowData, columnNames) {
				private static final long serialVersionUID = 1L;
				DefaultTableCellRenderer renderRight=new DefaultTableCellRenderer();            
				{//initializer block               
					renderRight.setHorizontalAlignment(SwingConstants.RIGHT);           
				}             
				@Override         
				public TableCellRenderer getCellRenderer(int arg0, int arg1) {                
					return renderRight;          
				} 
			};
			
			table.setFillsViewportHeight(true);

			@SuppressWarnings("unused")
			ExcelAdapter xcl = new ExcelAdapter(table);
			
			JScrollPane scrollPane = new JScrollPane(table);

			scrollPane.setBounds(0, 90, 218, 293);
			scrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			p.add(scrollPane);
			

		}

		public void clearTable() {
			((DefaultTableModel) table.getModel()).setRowCount(0);
		}
		
		public void addRow(double t, double phi, double omega) {
			Vector<String> row = new Vector<String>();
			
			row.addElement(String.format("%.3f", t));
			row.addElement(String.format("%.3f", Math.toDegrees(phi)));
			row.addElement(String.format("%.3f", Math.toDegrees(omega)));
			((DefaultTableModel) table.getModel()).addRow(row);
			
		}
		
		public double getG() {
			return dfG.getValue();
		}

		public double getLaenge() {
			return dfLaenge.getValue();
		}

		public double getPhi() {
			double phi = dfPhi.getValue();
			return Math.toRadians(phi); 
		}

		
	}

	

}
