package animation.pendel;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import gui.DecimalField;
import gui.MainFrame;
import gui.guiConsts;




public class DlgPendelMechanisch extends DlgPendel {
	
	
	private static final long serialVersionUID = 1L;
	

	@SuppressWarnings("unused")
	
	private PanelPendelMechanisch pendelPanel;
	private PanelPendelParameter parameterPanel;
	
	
	private final int DLG_PANEL_WIDTH = 950;
	private final int DLG_PANEL_HEIGHT = 520;
	
	private final int DRAWPANEL_X = 20;
	private final int DRAWPANEL_Y = 20;
	private final int DRAWPANEL_WIDTH = 440;
	private final int DRAWPANEL_HEIGHT = 440;
	
	private final int PARAM_PANEL_X = 480;
	private final int PARAM_PANEL_Y = 10;
	private final int PARAM_PANEL_WIDTH = 500;
	private final int PARAM_PANEL_HEIGHT = 400;
	
	private final int TABLE_WIDTH = 400;
	private final int TABLE_HEIGHT = 293;
	
	private final int X1 = PARAM_PANEL_X;
	private final int Y1 = PARAM_PANEL_HEIGHT + 35;
	private final int WBUT = 80;
	private final int HBUT = 25;
	
	private final double scaling = 0.01;  // 1m = 100 Pixel
	
	public DlgPendelMechanisch(MainFrame mainFrame) {
		
		this.mainFrame = mainFrame;
		setTitle("Mechanisches Pendel");
		setSize(DLG_PANEL_WIDTH,DLG_PANEL_HEIGHT);
		
		parameterPanel = new PanelPendelParameter();
		parameterPanel.setBounds(PARAM_PANEL_X, PARAM_PANEL_Y, PARAM_PANEL_WIDTH, PARAM_PANEL_HEIGHT);
		pBackground.add(parameterPanel);
		
		pendelPanel = new PanelPendelMechanisch(parameterPanel);
		pendelPanel.setBounds(DRAWPANEL_X, DRAWPANEL_Y, DRAWPANEL_WIDTH, DRAWPANEL_HEIGHT);
		pBackground.add(pendelPanel);	
		
		JButton butStart = new JButton("Start");
		butStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdStart();
			}});
		
		butStart.setBounds(X1,Y1,WBUT,HBUT);
		pBackground.add(butStart);
		
		JButton butStop = new JButton("Stop");
		butStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdStop();
			}});
		
		butStop.setBounds(X1+WBUT+10,Y1,WBUT,HBUT);
		pBackground.add(butStop);
		
		JButton butCancel = new JButton("Beenden");
		butCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				cmdCancel();
			}});
		butCancel.setBounds(X1+2*WBUT+20,Y1,WBUT+20,HBUT);
		
		pBackground.add(butCancel);
	}

	protected void cmdStart() {
		pendelPanel.startAnimation();
	}
	
	protected void cmdStop() {
		pendelPanel.stopAnimation();
	}
	
	protected void cmdCancel() {
		pendelPanel.stopAnimation();
		pendelPanel.setVisible(false);
		dispose();
	}

	private class PanelPendelMechanisch extends PanelPendel {

		private static final long serialVersionUID = 1L;
		private boolean dgl_init_dynamisch = false;
		private PanelPendelParameter parameterPanel;
		private double T;

		public PanelPendelMechanisch(PanelPendelParameter parameterPanel) {
			this.parameterPanel = parameterPanel;
			initDgl();
		}

		protected void paintComponent(Graphics g) {

			
			int len = (int) (Math.round(L/scaling));

			int xs = 0;
			int ys = 0;

			Graphics2D g2d;

			if (offscreen == null) {
				offscreen = createImage(WIDTH, HEIGHT);
				bg = offscreen.getGraphics();
			}

			if (bg != null) {
				g2d = (Graphics2D) bg;

				bg.clearRect(0, 0, WIDTH, HEIGHT);
				bg.setColor(Color.WHITE);
				bg.fillRect(0, 0, guiConsts.imageDrawPanelWidth,
						guiConsts.imageDrawPanelHeight);

				bg.setColor(Color.BLACK);

				bg.fillOval(WIDTH / 2 - 2, HEIGHT / 2 - 2, 4, 4);

				bg.setColor(Color.LIGHT_GRAY);
				// Turn on anti-aliasing.
				g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
						RenderingHints.VALUE_ANTIALIAS_OFF);

				// Create a rectangle.
				Shape shape1 = new Rectangle2D.Double(WIDTH / 2, HEIGHT / 2, 10.0,
						len);

				// Take the shape object and create a rotated version
				AffineTransform at1 = AffineTransform.getRotateInstance(-phi,
						WIDTH / 2 + 5, HEIGHT / 2);
				Shape shape2 = at1.createTransformedShape(shape1);

				// Take the shape object and create a translated version
				AffineTransform at2 = AffineTransform.getTranslateInstance(-5, 0);
				Shape shape3 = at2.createTransformedShape(shape2);
				g2d.fill(shape3);

				Schwerpunkt(WIDTH, HEIGHT, shape3);
				getMomentOfInertia(WIDTH, HEIGHT, WIDTH / 2, HEIGHT / 2, shape3,scaling,M);
				xs = (int) Math.round(pendeldaten.getXs());
				ys = (int) Math.round(pendeldaten.getYs());

				bg.setColor(Color.BLACK);
				bg.drawLine(xs - 2, ys, xs + 2, ys);
				bg.drawLine(xs, ys - 2, xs, ys + 2);

				Dgl();
				g.drawImage(offscreen, 0, 0, this);
			}
		}

		
		
		
		@Override
		protected void Dgl() {
			// Winkel des Fadenpendels (im Bogenmass)
			double dw; // Winkelbeschleunigung
			double s = L/2;
			if (dgl_init_dynamisch==false) {
				dgl_init_dynamisch=true;
				I = pendeldaten.getMassenTraegheitsmoment();
				T = Math.sqrt(I/(M * g * s)) * 2.0 * Math.PI;
				parameterPanel.setPeriod(T);
			}
				
			parameterPanel.addRow(
					t, phi, w, 
					(int) pendeldaten.getArea(),
					pendeldaten.getXs(), 
					pendeldaten.getYs(), 
					I);
			
			phi = phi + w*dt; // Winkel des Fadenpendels (im Bogenmass, Ruhelage phi=0)
			dw = (-M*g*s*Math.sin(phi)/I)*dt;
			w = w + dw; // w= Winkelgeschwindigkeit = dphi/dt
			
			t = t + dt;
		

		}

		@Override
		protected void initDgl() {
			M = parameterPanel.getMasse();
			g = parameterPanel.getG();
			L = parameterPanel.getLaenge();
			phi = parameterPanel.getPhi();
			w = 0.0;
			t = 0.0;
			dt = 0.05;
			dgl_init_dynamisch = false;
			
		}

		@Override
		protected void clearTable() {
			parameterPanel.clearTable();

		}

	}
	
	private class PanelPendelParameter extends PanelParameter{
		
		
		private static final long serialVersionUID = 1L;
		
		private JTable table;
		
		private DecimalField dfMasse; 
		private DecimalField dfLaenge; 
		private DecimalField dfPhi;
		private DecimalField dfPeriod;

		
		public PanelPendelParameter() {
			
			dfMasse = new DecimalField(0,3);
			dfLaenge = new DecimalField(0,3);
			dfPhi = new DecimalField(0,3);
			dfPeriod = new DecimalField(0,3);
		
			setParameter(this,dfMasse, "Masse [Kg]", 0, 10, 0.4,0.1,20);
			setParameter(this,dfLaenge, "Stablnge [m]", 0, 35, 1.0,0.1,2.0);
			setParameter(this,dfPhi, "Phi", 0, 60,  10, 0, 360);
			
			setDecimalfield(this,dfPeriod,"Periodendauer [s]",230,10,110,50);
			dfPeriod.setEditable(false);

			initTable(this);
		}
		
		
		public double getG() {
			return 9.81;
		}
		public double getMasse() {
			return dfMasse.getValue();
		}

		public double getLaenge() {
			return dfLaenge.getValue();
		}
		
		public double getPhi() {
			double phi = dfPhi.getValue();
			return Math.toRadians(phi); 
		}
		
		public void setPeriod(double t) {
			dfPeriod.setValue(t);
		}
		
		public void addRow(double t, double phi, double omega, 
				int area, double xs, double ys, double I) {
			Vector<String> row = new Vector<String>();
			
			row.addElement(String.format("%.3f", t));
			row.addElement(String.format("%.3f", Math.toDegrees(phi)));
			row.addElement(String.format("%.3f", Math.toDegrees(omega)));
			row.addElement(String.format("%d", area));
			row.addElement(String.format("%.3f", xs));
			row.addElement(String.format("%.3f", ys));
			row.addElement(String.format("%.3f", I));
			((DefaultTableModel) table.getModel()).addRow(row);
			
		}

		@Override
		protected void initTable(JPanel p) {
			columnNames = new Vector<String>();
			columnNames.addElement("t [s]");
			columnNames.addElement("Phi []");
			columnNames.addElement("Omega []");
			columnNames.addElement("Area [Pixel]");
			columnNames.addElement("Xs [Pixel]");
			columnNames.addElement("Ys [Pixel]");
			columnNames.addElement("I [Kg*m^2]");

			rowData = new Vector<Vector<String>>();
			table = new JTable(rowData, columnNames){            
				
				private static final long serialVersionUID = 1L;
				DefaultTableCellRenderer renderRight=new DefaultTableCellRenderer();            
				{//initializer block               
					renderRight.setHorizontalAlignment(SwingConstants.RIGHT);           
				}             
				@Override         
				public TableCellRenderer getCellRenderer(int arg0, int arg1) {                
					return renderRight;          
				}        
			};
			table.getColumnModel().getColumn(0).setPreferredWidth(50);
			table.getColumnModel().getColumn(1).setPreferredWidth(60);
			table.getColumnModel().getColumn(2).setPreferredWidth(80);
			table.getColumnModel().getColumn(3).setPreferredWidth(80);
			table.getColumnModel().getColumn(4).setPreferredWidth(70);
			table.getColumnModel().getColumn(5).setPreferredWidth(70);
			
			table.setFillsViewportHeight(true);

			JScrollPane scrollPane = new JScrollPane(table);

			scrollPane.setBounds(0, 90, TABLE_WIDTH, TABLE_HEIGHT);
			scrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			p.add(scrollPane);
			
		}
		
		public void clearTable() {
			((DefaultTableModel) table.getModel()).setRowCount(0);
		}
	}

}
