package animation.pendel;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Shape;
import java.lang.Math;


import gui.ImagePanel;

public abstract class PanelPendel extends ImagePanel  {

	private static final long serialVersionUID = 1L;

	protected final int WIDTH = 440;
	protected final int HEIGHT = 440;
	
	protected Image offscreen;
	protected Graphics bg;
	
	protected DrawThread drawThread;
	protected boolean running;
	protected boolean bStartAnimation;
	
	protected double M;  	// Masse
	protected double I;  	// Traegheitsmoment	
	protected double g;  	// Fallbeschleunigung	
	protected double L;		// Pendellaenge
	protected double phi;	// Aktueller Winkel
	protected double w;		// Winkelgeschwindigkeit
	protected double dw;	// delta "
	protected double t;		// Zeit
	protected double dt;	// Delta t
	
	protected PendelDaten pendeldaten;
	
	public PanelPendel() {
		
		setBackground(Color.WHITE);
		pendeldaten = new PendelDaten();
		offscreen = null;
		drawThread = new DrawThread();
		running = true;
		bStartAnimation = false;
		drawThread.start();
	}
	
	public void startAnimation() {
		clearTable();
		initDgl();
		bStartAnimation = true;
	}
	
	public void stopAnimation() {
		bStartAnimation = false;
	}

	
	public void update(Graphics g) {
		paintComponent(g);
	}

 
	protected abstract void clearTable();
	
	protected abstract void initDgl();
	
	protected abstract void Dgl();
	
	
	protected class DrawThread extends Thread {
		@Override
		public void run() {
			// loop until told to stop
			while (running) {
				if (bStartAnimation) {
					repaint();
				}
				
				try {
					Thread.sleep(100);
				} catch (InterruptedException e) {
					System.out.println(e);
				}
			}
		}
		
		 public void destroy() 
         { 
              running = false;    
         } 
	}


	protected void Schwerpunkt(int width, int height, Shape shape) {
	
		double area = 0.0;
	    double xs = 0.0;
	    double ys = 0.0;
	    
	    for (int x=0; x<width; x++) {
	    	for (int y=0; y<height; y++) {
	    		if (shape.contains(x, y)) {
	    			xs += x;
	    			ys += y;
	    			area++;
	    		}
	    	}
	    }
	    xs /= area;
	    ys /= area;
		pendeldaten.setXs(xs);
		pendeldaten.setYs(ys);
		pendeldaten.setArea(area);
	}
	
	protected void getMomentOfInertia(int width, int height,double xz, double yz, Shape shape,double scaling, double mass) {
		double I = 0.0;
		int area = 0;
		for (int x=0; x<width; x++) {
		    for (int y=0; y<height; y++) {
		    	if (shape.contains(x, y)) {
		    		I += Math.pow((x-xz)*scaling, 2)+Math.pow((y-yz)*scaling, 2);
		    		area += 1;
		    	}
		    }
		}
		pendeldaten.setMomentOfInertia(I * mass /area);
	}
	
	protected class PendelDaten {
		double xs;
		double ys;
		double s;
		double xz;
		double yz;
		double area;
		double mass;
		double scaling;
		double g;
		double momentOfInertia;
		
		public PendelDaten() {
			xs = 0.0;
			ys = 0.0;
			xz = 0.0;
			yz = 0.0;
			area = 0.0;
			mass = 0.0;
			momentOfInertia = 0.0;
		}
		
		public void setMomentOfInertia(double value) {
			momentOfInertia = value;;
		}
		
		public double getMassenTraegheitsmoment() {
			return momentOfInertia;
		}
		
		public double getXs() {
			return xs;
		}
		public void setXs(double xs) {
			this.xs = xs;
		}
		public double getYs() {
			return ys;
		}
		public void setYs(double ys) {
			this.ys = ys;
		}
		public double getArea() {
			return area;
		}
		public void setArea(double area) {
			this.area = area;
		}

		public double getXz() {
			return xz;
		}

		public void setXz(double xz) {
			this.xz = xz;
		}

		public double getYz() {
			return yz;
		}

		public void setYz(double yz) {
			this.yz = yz;
		}

		public double getScaling() {
			return scaling;
		}

		public void setScaling(double scaling) {
			this.scaling = scaling;
		}

		public double getMasse() {
			return mass;
		}

		public void setMass(double mass) {
			this.mass = mass;
		}

		public double getSchwerpunktradius() {
			// Abstand Schwerpunkt von Zentrum (in m)
			return scaling * Math.sqrt(Math.pow(xz-xs, 2) + Math.pow(yz-ys,2));
		}
		
		public double getPhi() {
			return Math.atan2(yz-ys,xz-xs);
		}

		public void setS(double s) {
			this.s = s;
		}

		public double getGravitationskonstante() {
			return g;
		}

		public void setG(double g) {
			this.g = g;
		}
		
	}

}
