package animation.spiralrad;

import static java.lang.Math.cos;
import static java.lang.Math.sin;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.Vector;

import animation.pendel.PanelPendel;

public class PanelSpiralrad extends PanelPendel {
	private static final long serialVersionUID = 1L;

	private PanelSpiralradParameter parameterPanel;

	private SpiralradDaten spiralradDaten;

	private boolean init_dynamisch;

	private int innenRadius;
	private int aussenRadius;
	private int anzahlSpiralen;
	private int wassermenge;
	private double teilung;
	private double startwert_phi;
	
	private  final Point rad_zentrum = new Point(WIDTH/2,HEIGHT/2);
	private  Point spiral_zentrum;
	
	private Vector<Spirale> spiralen;
	

	public PanelSpiralrad(PanelSpiralradParameter parameterPanel) {
		this.parameterPanel = parameterPanel;
		init_dynamisch = false;
		spiral_zentrum = new Point(0,0);
		spiralradDaten = new SpiralradDaten();
		spiralen = new Vector<Spirale>();
		getParameter();
	}

	private void getParameter() {
		innenRadius = parameterPanel.getInnenRadius();
		aussenRadius = parameterPanel.getAussenRadius();
		teilung = parameterPanel.getTeilung();
		startwert_phi = parameterPanel.getPhi();
		anzahlSpiralen = parameterPanel.getAnzahlSpiralen();
		wassermenge = parameterPanel.getWassermenge();
	}

	public void reset() {
		// Der Aufrufer von reset() ruft am Schluss repaint() auf.
		// Ueber repaint() wird ueber die Java VM paintComponent(Graphics gr)
		// aufgerufen. 
		// Weil init_dynamisch = false hier gesetzt ist, wird in paintComponent(...)
		// der Rest der Initialisierung vorgenommen.
		getParameter();
		init_dynamisch = false;
	}
	
	private void zeichneRad(Graphics2D bg2) {
		bg2.setColor(Color.red);
		int h = 2 * aussenRadius;
		int w = 2 * aussenRadius;
		int x = (WIDTH-w) / 2;
		int y = (HEIGHT-h) / 2;
		bg2.setColor(Color.black);
		bg2.drawArc(x,y,w,h,0,360);
	}
	
	/*
	private void zeichneEinlass(Graphics2D bg2, double phi) {
		int h = 2 * innenRadius;
		int w = 2 * innenRadius;
		bg2.setColor(Color.black);
		int x = (WIDTH-w) / 2;
		int y = (HEIGHT-h) / 2;
		double phi1 = phi * -1.0;
		int alpha = (int)Math.round(Math.toDegrees(phi1)); 
		bg2.drawArc(x,y,w,h,alpha,180);
		double dx0 = WIDTH/2-cos(phi1)*innenRadius;
		double dy0 = HEIGHT/2+sin(phi1)*innenRadius;
		
		double dx1 = WIDTH/2+cos(phi1)*innenRadius;
		double dy1 = HEIGHT/2-sin(phi1)*innenRadius;
		
		bg2.drawLine((int)dx0, (int)dy0, (int)dx1, (int)dy1);
		
	}
	*/
	
	private void zeichneVordergrund(Graphics2D bg2, double phi) {
		
		bg2.setColor(Color.yellow);
		bg2.drawLine(WIDTH/2, 1, WIDTH/2, HEIGHT-1);
		
		bg2.setColor(Color.blue);
		bg2.drawLine(1, HEIGHT/2, WIDTH-1, HEIGHT/2);
	}
	
	

	private void updateSpiralrad(BufferedImage img,
			Graphics2D bg2, double phi,
			SpiralradDaten spiralradDaten) {
		zeichneRad(bg2);
		
		double winkel = 2.0*Math.PI / anzahlSpiralen;
		int i = 0;
		for (Spirale spirale : spiralen) {
			spirale.zeichne(bg2,phi + i*winkel);
			i++;
		}

		zeichneVordergrund(bg2, phi);
	}

	protected void paintComponent(Graphics gr) {

		Graphics2D g2 = (Graphics2D) gr;

		// Zuerst in ein BufferedImage zeichnen.
		// Ueber dieses kann ggf. eine GIF Animation erstellt werden.
		BufferedImage img = new BufferedImage(WIDTH, HEIGHT,
				BufferedImage.TYPE_INT_RGB);
		Graphics bg = img.createGraphics();
		Graphics2D bg2 = (Graphics2D) bg;

		bg2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		// Panel zeichnen
		bg2.clearRect(0, 0, WIDTH, HEIGHT);
		bg2.setColor(Color.WHITE);
		bg2.fillRect(0, 0, WIDTH, HEIGHT);

		if (init_dynamisch == false) {
			phi = startwert_phi;
			spiral_zentrum.x = (int)Math.round(rad_zentrum.x - cos(phi) * innenRadius);
			spiral_zentrum.y = (int)Math.round(rad_zentrum.y - sin(phi) * innenRadius);
			spiralen.clear();
			for (int i=0; i<anzahlSpiralen; i++) {
				spiralen.add(new Spirale( 
						innenRadius, 
						aussenRadius, 
						rad_zentrum,
						spiral_zentrum,
						teilung,
						wassermenge));
			}
			init_dynamisch = true;
		}

		updateSpiralrad(img, bg2, phi, spiralradDaten);

		// Ggf. neues Image ins animierte GIF eintragen
		parameterPanel.writeImage(img);

		// Image auf dem Bildschirm darstellen
		g2.drawImage(img, 0, 0, null);

		phi = phi - 0.05; 

	}

	@Override
	protected void clearTable() {
		parameterPanel.clearTable();

	}

	@Override
	protected void Dgl() {
		// TODO Auto-generated method stub

	}

	@Override
	protected void initDgl() {
		// TODO Auto-generated method stub

	}

}
