package animation.spiralrad;

import gui.DecimalField;
import gui.ImageFilter;
import imaging.GifSequenceWriter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.imageio.stream.FileImageOutputStream;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import printing.DruckAuftragSpirale;

import animation.pendel.PanelParameter;

public class PanelSpiralradParameter extends PanelParameter {
	
	private static final long serialVersionUID = 1L;
	private final int TABLE_WIDTH = 400;
	private final int TABLE_HEIGHT = 243;
	

	
	private final int INNEN_RADIUS = 0;
	private final int AUSSEN_RADIUS = 200;
	private final double TEILUNG = 0.2;
	private final double PHI = 180.0;
	private final int ANZAHL_SPIRALEN = 12;
	private final int WASSERMENGE = 500;
	
	
	private JTable table;

	private PanelSpiralrad panelSpiralrad;

	private DecimalField dfInnenRadius = new DecimalField(0,0); // int
	private DecimalField dfAussenRadius = new DecimalField(0,0); // int
	private DecimalField dfTeilung = new DecimalField(0,1); // double
	private DecimalField dfPhi = new DecimalField(0,1);
	private DecimalField dfAnzahlSpiralen = new DecimalField(0,0); // int
	private DecimalField dfWassermenge = new DecimalField(0,0); // int
	
	private JButton butCreateGIF;
	private JButton butSetParameter;
	private JButton butDruckeSpiralrad;
	
	private boolean bGifErzeugen;
	private GifSequenceWriter writer;
	private ImageOutputStream output;
	
	


	public PanelSpiralradParameter() {
		
		butCreateGIF = new JButton("GIF erzeugen");
		butCreateGIF.setBounds(290,10,110,20);
		add(butCreateGIF);
		butCreateGIF.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0) {
				cmdGifErzeugen();
			}});

		butSetParameter = new JButton("Set Param.");
		butSetParameter.setBounds(290,35,110,20);
		add(butSetParameter);
		butSetParameter.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0) {
				cmdSetParameter();
			}});
		
		butDruckeSpiralrad = new JButton("Drucken");
		butDruckeSpiralrad.setBounds(290,60,110,20);
		add(butDruckeSpiralrad);
		butDruckeSpiralrad.addActionListener(new ActionListener(){

			@Override
			public void actionPerformed(ActionEvent arg0) {
				cmdSpiralradDrucken();
			}});


		
		setInitialParameters();
		
		bGifErzeugen = true;

		initTable(this);
	}

	public void setSpiralradPanel(PanelSpiralrad panelSpiralrad) {
		this.panelSpiralrad = panelSpiralrad;
	}
	
	private void setInitialParameters() {	
		setParameter(this, dfInnenRadius, "Innenradius", 0, 10, INNEN_RADIUS, 0, 150);
		setParameter(this, dfAussenRadius, "Aussenradius", 0, 35, AUSSEN_RADIUS, 180, 220);
		setParameter(this, dfTeilung, "Umdrehungen", 0, 60, TEILUNG, 0.1, 10);
		setParameter(this, dfPhi, "Winkel", 0, 85, PHI, 0, 360);
		setParameter(this, dfAnzahlSpiralen, "Anz. Spiralen", 0, 110, ANZAHL_SPIRALEN, 1, 20);
		setParameter(this, dfWassermenge, "Wassermenge", 0, 135, WASSERMENGE, 100, 5000);
	}
	
	private void resetGIFWriter() {
		if (writer != null) {
			try {
				writer.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (output != null) {
			try {
				output.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void reset() {
		//setInitialParameters();
		
		clearTable();
		
		butCreateGIF.setText("GIF erzeugen");
		
		resetGIFWriter();
		bGifErzeugen = true;
		
	}
	
	public double getG() {
		return 9.81;
	}


	public double getPhi() {
		double phi = dfPhi.getValue();
		return Math.toRadians(phi);
	}

	public void writeImage(BufferedImage img) {
		if (writer == null) return;
		if (output == null) return;
		if (img == null) return;
		if (bGifErzeugen==true) return;
		try {
			writer.writeToSequence(img);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public boolean mnu_FileOpenAs() {

		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(new File("Spiralrad-443.gif"));
		ImageFilter imageFilter = new ImageFilter();
		chooser.addChoosableFileFilter(imageFilter);
		chooser.setAcceptAllFileFilterUsed(false);

		int rval = chooser.showSaveDialog(this);
		if (rval == JFileChooser.APPROVE_OPTION) {
			File fileToSave = chooser.getSelectedFile();
			if (imageFilter.accept(fileToSave)) {
				if (fileToSave.exists()) {
					int response = JOptionPane.showConfirmDialog(null,
							"Overwrite existing file?", "Confirm Overwrite",
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (response == JOptionPane.CANCEL_OPTION)
						return false;
				}
				
				try {
					
					output = new FileImageOutputStream(fileToSave);
					writer = new GifSequenceWriter(
							output, BufferedImage.TYPE_INT_RGB, 50, true);
					return true;
					
					
				} catch (IOException ex) {
					JOptionPane.showMessageDialog(this, ""
							+ "File cannot be saved", "File Error",
							JOptionPane.ERROR_MESSAGE);
					
				}
			} else {
				JOptionPane.showMessageDialog(this, ""
						+ "File cannot be saved (wrong extension).",
						"File Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		return false;
	}

	public void addRow(double t, double phi, double omega, double area,
			double xs, double ys, double I) {
		Vector<String> row = new Vector<String>();

		row.addElement(String.format("%.3f", t));
		row.addElement(String.format("%.3f", Math.toDegrees(phi)));
		row.addElement(String.format("%.3f", Math.toDegrees(omega)));
		row.addElement(String.format("%.3f", area));
		row.addElement(String.format("%.3f", xs));
		row.addElement(String.format("%.3f", ys));
		row.addElement(String.format("%.3f", I));
		((DefaultTableModel) table.getModel()).addRow(row);

	}

	@Override
	protected void initTable(JPanel p) {
		columnNames = new Vector<String>();
		columnNames.addElement("t [s]");
		columnNames.addElement("Phi []");
		columnNames.addElement("Omega []");
		columnNames.addElement("Area [m^2]");
		columnNames.addElement("Xs [m]");
		columnNames.addElement("Xs [m]");
		columnNames.addElement("I [Kg*m^2]");

		rowData = new Vector<Vector<String>>();
		table = new JTable(rowData, columnNames) {

			private static final long serialVersionUID = 1L;
			DefaultTableCellRenderer renderRight = new DefaultTableCellRenderer();
			{// initializer block
				renderRight.setHorizontalAlignment(SwingConstants.RIGHT);
			}

			@Override
			public TableCellRenderer getCellRenderer(int arg0, int arg1) {
				return renderRight;
			}
		};
		table.getColumnModel().getColumn(0).setPreferredWidth(50);
		table.getColumnModel().getColumn(1).setPreferredWidth(70);
		table.getColumnModel().getColumn(2).setPreferredWidth(80);
		table.getColumnModel().getColumn(3).setPreferredWidth(80);
		table.getColumnModel().getColumn(4).setPreferredWidth(70);
		table.getColumnModel().getColumn(5).setPreferredWidth(70);

		table.setFillsViewportHeight(true);

		JScrollPane scrollPane = new JScrollPane(table);

		scrollPane.setBounds(0, 160, TABLE_WIDTH, TABLE_HEIGHT);
		scrollPane
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		//p.add(scrollPane);

	}
	
	private void cmdSetParameter() {
		panelSpiralrad.reset();
		panelSpiralrad.repaint();
	}

	public void cmdGifErzeugen() {
		if (bGifErzeugen==true) {
			
			butCreateGIF.setText("GIF beenden");
			if (mnu_FileOpenAs()==true) {
				bGifErzeugen = false;
			}
		}
		else {
			bGifErzeugen = true;
			butCreateGIF.setText("GIF erzeugen");
			resetGIFWriter();
		}
	}
	
	private void cmdSpiralradDrucken() {
		SpiralradDaten spiralradDaten = new SpiralradDaten();
		spiralradDaten.setInnenRadius((int)dfInnenRadius.getValue());
		spiralradDaten.setAussenRadius((int)dfAussenRadius.getValue());
		spiralradDaten.setAnzahlSpiralen((int)dfAnzahlSpiralen.getValue());
		spiralradDaten.setTeilung(dfTeilung.getValue());
		new DruckAuftragSpirale(null,spiralradDaten);
	}
	
	public void clearTable() {
		((DefaultTableModel) table.getModel()).setRowCount(0);
	}

	public int getInnenRadius() {
		return (int)dfInnenRadius.getValue();
	}

	public int getAussenRadius() {
		return (int)dfAussenRadius.getValue();
	}
	
	public int getAnzahlSpiralen() {
		return (int)dfAnzahlSpiralen.getValue();
	}
	
	public int getWassermenge() {
		return (int)dfWassermenge.getValue();
	}

	public double getTeilung() {
		return dfTeilung.getValue();
	}
	
}
