package animation.spiralrad;

import static java.lang.Math.cos;
import static java.lang.Math.sin;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.Vector;

public class Spirale {

	private Vector<Point> spiral_punkte_roh;
	private Vector<Point> spiral_punkte;
	private Vector<Integer> tiefste_spiral_punkte;

	private Point rad_zentrum;
	private double innenRadius;
	private double aussenRadius;
	private double teilung;
	private int wassermenge;

	public Spirale(double innenRadius,
			double aussenRadius, Point rad_zentrum, Point spiral_zentrum,
			double teilung,
			int wassermenge) {
		this.rad_zentrum = rad_zentrum;
		this.innenRadius = innenRadius;
		this.aussenRadius = aussenRadius;
		this.teilung = teilung;
		this.spiral_punkte_roh = new Vector<Point>();
		this.spiral_punkte = new Vector<Point>();
		this.tiefste_spiral_punkte = new Vector<Integer>();
		this.wassermenge = wassermenge;
	}

	private void line(Graphics2D bg2, int x0, int y0, int x1, int y1) {
		//
		// Bresenham Algorithmus
		// http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
		// 
		int dx = Math.abs(x1 - x0);
		int dy = Math.abs(y1 - y0);
		int sx = (x0 < x1) ? 1 : -1;
		int sy = (y0 < y1) ? 1 : -1;
		int err = dx - dy;
		int e2;
		
		int xn0, yn0;
		int xn1, yn1;

		int size;

		while (true) {
			// Vermeiden, dass die gleiche Punktkoordinate
			// mehrfach hintereinander im Punkte-Vektor abgelegt wird.
			// Kann infolge Rundungsfehlern beim Berechnen der 
			// Spirale vorkommen.
			size = spiral_punkte_roh.size();
			if (size > 0) {
				xn0 = spiral_punkte_roh.get(size-1).x;
				yn0 = spiral_punkte_roh.get(size-1).y;
				xn1 = x0;
				yn1 = y0;
			
				if ((xn1 != xn0) || (yn1 != yn0)) {
					if (!((xn1 == xn0) && (yn1 == yn0))) {
						spiral_punkte_roh.add(new Point(x0, y0));
					}
				}
			}
			else {
				spiral_punkte_roh.add(new Point(x0, y0));
			}
			
			// Hier beginnt der eigentlich Bresenham-Algorithmus
			// zum Zeichnen einer Geraden zwischen x0,y0 und x1,y1.
			if ((x0 == x1) && (y0 == y1)) {
				break;
			}
			e2 = 2 * err;
			if (e2 > -dy) {
				err = err - dy;
				x0 = x0 + sx;
			}
			if (e2 < dx) {
				err = err + dx;
				y0 = y0 + sy;
			}
		}
	}	
	
	private void bereinigeSpiralpunkte(Graphics2D bg2) {
		// Folgende Pixel im Kurvenzug der Spirale loeschen:
		// 
		// Vorher:  Ersatz:
		// 0          0
		// 00          0
		//
		//  0          0
		// 00         0
		
		int numPunkte = spiral_punkte_roh.size();
		int x0, y0;
		int x1, y1;
		int x2, y2;
		
		for (int i=2; i< numPunkte; i++) {
			x0 = spiral_punkte_roh.get(i-2).x;
			y0 = spiral_punkte_roh.get(i-2).y;
			x1 = spiral_punkte_roh.get(i-1).x;
			y1 = spiral_punkte_roh.get(i-1).y;
			x2 = spiral_punkte_roh.get(i).x;
			y2 = spiral_punkte_roh.get(i).y;
			
			boolean ok = true;
			if ((y2 == (y1-1)) && (y1 == y0) && (x2 == x1)) {
				ok = false;
			}
			if ((y0 == (y1-1)) && (y2 == y1) && (x0 == x1)) {
				ok = false;
			}
			if (ok == true) {
				spiral_punkte.add(spiral_punkte_roh.get(i-1));
				bg2.drawLine(x1, y1, x1, y1);
			}
		}
	}
	
	private Point pLinksSuchen(int p, int y) {
		int numPunkte = spiral_punkte.size();
		int i = p;
		while (i < numPunkte) {
			if (spiral_punkte.get(i).y==y) {
				return (new Point(spiral_punkte.get(i).x,y));
			}
			i++;
		}
		return null;
	}
	
	private Point pRechtsSuchen(int p, int y) {
		int i = p-1;
		while (i >= 0) {
			if (spiral_punkte.get(i).y==y) {
				return (new Point(spiral_punkte.get(i).x,y));
			}
			i--;
		}
		return null;
	}
	
	private void zeichne_wassereinschluesse(Graphics2D bg2, int wassermenge) {
		
		Point pLinks, pRechts;
		int xLinks, xRechts;
		int x,y;
		int numWasserPixel;
		int wassermenge_aktuell;
		
		bg2.setColor(Color.blue);
		for (Integer p :tiefste_spiral_punkte) {
			
			y = spiral_punkte.get(p).y;
			wassermenge_aktuell = wassermenge;
			
			while (wassermenge_aktuell > 0) {
				// Das erste Pixel links nach dem flachen Teil unten
				pLinks = pLinksSuchen(p,y);
				pRechts = pRechtsSuchen(p,y);
			
				if ((pLinks == null) || (pRechts == null)) {
					break;
				}
				else {
					xLinks = pLinks.x;
					xRechts = pRechts.x;
					
					numWasserPixel = (xRechts-1) - (xLinks+1) +1;
					x = xLinks+1;
					while ((numWasserPixel > 0) && (wassermenge_aktuell >0)) {
						bg2.drawLine(x, y, x, y);
						numWasserPixel--;
						wassermenge_aktuell--;
						x++;
					}	
					y--;
				}
			}
		}
	}
	
	private void sucheTiefstePunkte(Graphics2D bg2) {

		final int FALLEND = 0;
		final int FLACH = 1;
		final int STEIGEND = 2;
		final int FALSCH = 3;
		
		
		final int WARTEN_AUF_FALLEND = 0;
		final int WARTEN_AUF_FLACH_NACH_FALLEND = 1;
		final int WARTEN_AUF_STEIGEND_NACH_FLACH = 2;
		
		
		int status = WARTEN_AUF_FALLEND; 
		int event;
		
		int x0;
		int y0;
		int x1;
		int y1;
		
		// Statusmaschine
		// Gesucht wird das erste Pixel nach fallender Kurve 
		// und nach dem flachem Teil unten.
		int numPoints = spiral_punkte.size();
		
		for (int n = 1; n < numPoints; n++) {
			
			x0 = spiral_punkte.get(n-1).x;
			y0 = spiral_punkte.get(n-1).y;
			x1 = spiral_punkte.get(n).x;
			y1 = spiral_punkte.get(n).y;
			
			event = FALSCH;
			if (x1 < x0) {
				// y-Koordinaten: Groessere Werte bedeuten tiefere Punkte
				if (y1>y0) {
					event = FALLEND;
				}
				if (y1==y0) {
					event = FLACH;
				}
				if (y1<y0) {
					event = STEIGEND;
				}
			}
			
			switch (status) {
			
			case WARTEN_AUF_FALLEND:
				switch (event) {
				case FALLEND:
					status = WARTEN_AUF_FLACH_NACH_FALLEND;
					break;
				default:
					status = WARTEN_AUF_FALLEND;
					break;
				}
				break;
				
			case WARTEN_AUF_FLACH_NACH_FALLEND:
				switch (event) {
				case FALLEND:
					status = WARTEN_AUF_FLACH_NACH_FALLEND;
					break;
				case FLACH:
					status = WARTEN_AUF_STEIGEND_NACH_FLACH;
					break;
				default:
					status = WARTEN_AUF_FALLEND;
					break;
				}
				break;
			case WARTEN_AUF_STEIGEND_NACH_FLACH:
				switch (event) {
				case FALLEND:
					status = WARTEN_AUF_FLACH_NACH_FALLEND;
				case FLACH:
					status = WARTEN_AUF_STEIGEND_NACH_FLACH;
					break;
				case STEIGEND:
					// Koordinate des ersten Pixels in steigender Richtung nach links
					// nach dem unteren flachen Teil zufuegen.
					// y ist 1 Pixel UEBER dem flachen Teil.
					tiefste_spiral_punkte.add(n);
					status = WARTEN_AUF_FALLEND;
					break;
				default:
					status = WARTEN_AUF_FALLEND;
					break;
				}
				break;
			}
		}
	}

	public void zeichne(Graphics2D bg2, double phi) {

		spiral_punkte_roh.clear();
		spiral_punkte.clear();
		tiefste_spiral_punkte.clear();

		bg2.setColor(Color.black);

		double radius = innenRadius;
		int steps = (int) Math.round(360.0 * teilung);
		double deltaRadius = ((double) (aussenRadius - innenRadius)) / steps;
		

		//spiral_zentrum.x = (int)Math.round(rad_zentrum.x - cos(phi) * radius);
		//spiral_zentrum.y = (int)Math.round(rad_zentrum.y - sin(phi) * radius);
		
		double spirale_X_alt = rad_zentrum.x - cos(phi) * radius;
		double spirale_Y_alt = rad_zentrum.y - sin(phi) * radius;

		
		for (int n = 0; n < steps; n++) {

			double alpha = Math.toRadians(n) + phi;
			
			
			double xneu = rad_zentrum.x - cos(alpha) * radius;
			double yneu = rad_zentrum.y - sin(alpha) * radius;

			int x0 = (int) Math.round(spirale_X_alt);
			int y0 = (int) Math.round(spirale_Y_alt);
			int x1 = (int) Math.round(xneu);
			int y1 = (int) Math.round(yneu);

			line(bg2, x0, y0, x1, y1);
			
			spirale_X_alt = xneu;
			spirale_Y_alt = yneu;
			radius = radius + deltaRadius;
		}

		bereinigeSpiralpunkte(bg2);
		sucheTiefstePunkte(bg2);
		zeichne_wassereinschluesse(bg2,wassermenge);

	}

	public Vector<Point> getTiefstePunkte() {
		return null; //tiefste_spiral_punkte;
	}

	public Point getAnfang() {
		return spiral_punkte_roh.get(0);
	}
}
