package gui;

import javax.swing.JTextField;

import java.awt.Toolkit;
import java.text.*;

public class DecimalField extends JTextField {

	private static final long serialVersionUID = 1L;

	private NumberFormat format;

	/** Wert, welcher bei einer Fehlerhaften Eingabe wiederhergestellt wird. */
	private double retVal = 0;
	private double minValue;
	private double maxValue;

	private boolean bLimitTest = false;

	/**
	 * DecimalField constructor
	 * 
	 * @param spannungsvektorWert
	 *            Mit diesem Wert wird das Feld initialisiert
	 * @param columns
	 *            bestimmt die Breite des JTextFields (unter Bercksichtigung
	 *            von <code>f</code>)
	 * @param f
	 *            java.text.NumberFormat
	 */
	public DecimalField(double value, NumberFormat f) {
		setHorizontalAlignment(JTextField.RIGHT);
		format = f;
		bLimitTest = false;
		setValue(value);
	}

	public DecimalField(double value, int maxFractionDigits) {
		setHorizontalAlignment(JTextField.RIGHT);
		// setDocument(new FormattedDocument(f));
		NumberFormat f = NumberFormat.getNumberInstance();
		f.setMaximumFractionDigits(maxFractionDigits);
		format = f;
		bLimitTest = false;
		setValue(value);
	}

	/**
	 * Wert aus Textfeld holen
	 * 
	 * @return den im Texfeld dargestellten <code>float</code>
	 */
	public double getValue() {
		// retVal enthaelt den letzten gueltigen Wert.
		// retVal wird nur erneuert, wenn der neue gelesene Wert gueltig ist.
		try {
			double value = format.parse(getText()).doubleValue();
			if (bLimitTest) {
				if ((value >= minValue) && (value <= maxValue)) {
					retVal = value;
				} else {
					if (value < minValue) {
						setText(format.format(minValue));
						retVal = minValue;
					}
					if (value > maxValue) {
						setText(format.format(maxValue));
						retVal = maxValue;
					}
					Toolkit.getDefaultToolkit().beep();
				}
			} else {
				retVal = value;
			}
		} catch (ParseException e) {
			Toolkit.getDefaultToolkit().beep();
			setValue(retVal);
		}
		return retVal;
	}

	public void setLimits(double minValue, double maxValue) {
		this.minValue = minValue;
		this.maxValue = maxValue;
		bLimitTest = true;
	}

	public void setValue(String text) {
		text.trim();

		try {
			double value = Double.valueOf(text);
			if (bLimitTest) {
				if ((value >= minValue) && (value <= maxValue)) {
					setText(format.format(value));
					retVal = value;
				} else {
					Toolkit.getDefaultToolkit().beep();
				}
			} else {
				setText(format.format(value));
				retVal = value;
			}
		} catch (NumberFormatException e) {
			Toolkit.getDefaultToolkit().beep();
		}

	}

	public void setValue(double value) {
		if (bLimitTest) {
			if ((value >= minValue) && (value <= maxValue)) {
				setText(format.format(value));
				retVal = value;
			} else {
				Toolkit.getDefaultToolkit().beep();
			}
		} else {
			setText(format.format(value));
			retVal = value; // wichtiger Zusatz, wegen ParseException
		}
	}

}
