package gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.border.Border;

public class ImageDrawPanel extends ImagePanel implements MouseListener,
MouseMotionListener {

	private static final long serialVersionUID = 1L;

	private BufferedImage rotatedImage;
	private boolean rotate;
	private Cursor savedCursor;
	private ImageBasePanel imageBasePanel;

	AffineTransform affineTransform;

	private Vector<CenterOfArea> centersOfArea;

	public ImageDrawPanel(ImageBasePanel imageBasePanel) {
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.imageBasePanel = imageBasePanel;
		
		//this.imageBasePanel.add(this);

		this.setLayout(null);
		this.setBounds(20, 20, guiConsts.imageDrawPanelWidth,
				guiConsts.imageDrawPanelHeight);

		Border blackline = BorderFactory.createLineBorder(Color.black);
		setBorder(blackline);

		affineTransform = new AffineTransform();
		affineTransform.setToTranslation(1, 1);
		centersOfArea = new Vector<CenterOfArea>();

	}
	


	public Vector<CenterOfArea> getCentersOfArea() {
		return centersOfArea;
	}

	public void setImage(BufferedImage image) {
		this.image = image;
		repaint();
	}

	public void setRotatedImage(BufferedImage im) {
		rotatedImage = im;
		rotate = true;
	}

	protected void paintComponent(Graphics g) {

		Graphics2D g2d = (Graphics2D) g;
		int x;
		int y;

		g.setColor(Color.WHITE);
		g.fillRect(0, 0, guiConsts.imageDrawPanelWidth,
				guiConsts.imageDrawPanelHeight);

		if (rotate == true) {
			rotate = false;
			if (rotatedImage == null)
				return;
			g.drawImage(rotatedImage, 0, 0, null);
			return;
		}

		if (this.image != null) {
			g.drawImage(image, 0, 0, null);

			for (CenterOfArea centerOfArea : centersOfArea) {
				g2d.setColor(centerOfArea.getColorToDraw());
				x = centerOfArea.getX();
				y = centerOfArea.getY();
				g2d.drawLine(x - 10, y, x + 10, y);
				g2d.drawLine(x, y + 10, x, y - 10);
				g2d.drawOval(x - 10, y - 10, 20, 20);
				if (centerOfArea.isTotal()) {
					g2d.setColor(Color.WHITE);
					g2d.drawLine(x - 9, y, x + 9, y);
					g2d.drawLine(x, y + 9, x, y - 9);
					g2d.drawOval(x - 9, y - 9, 18, 18);
					g2d.setColor(Color.BLUE);
					g2d.drawLine(x - 11, y, x + 11, y);
					g2d.drawLine(x, y + 11, x, y - 11);
					g2d.drawOval(x - 11, y - 11, 22, 22);
				}
			}
		}
	}

	public BufferedImage captureActualImage() {
		BufferedImage img = new BufferedImage(image.getWidth(), image
				.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D g2 = img.createGraphics();
		this.paint(g2);
		return img;
	}

	public void mouseEntered(MouseEvent arg0) {
		savedCursor = getCursor();
		setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		setCursor(savedCursor);
	}
	
	@Override
	public void mouseClicked(MouseEvent arg0) {
		if (image == null)
			return;
		Point p = new Point();
		p.x = arg0.getX();
		p.y = arg0.getY();
		if (p.x >= image.getWidth())
			return;
		if (p.y >= image.getHeight())
			return;
		int rgb = image.getRGB(p.x, p.y);
		Color color = new Color(rgb);
		imageBasePanel.setActualPixelColor(p, color);
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		if (image == null)
			return;
		Point p = new Point();
		p.x = arg0.getX();
		p.y = arg0.getY();
		if (p.x >= image.getWidth())
			return;
		if (p.y >= image.getHeight())
			return;
		int rgb = image.getRGB(p.x, p.y);
		Color color = new Color(rgb);
		imageBasePanel.showActualPixelColor(p, color);
	}
	
	@Override
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	@Override
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	

}
