package gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.Properties;

import javax.imageio.*;
import javax.swing.*;

import animation.spiralrad.DlgSpiralradPhysikalischeDaten;
import animation.balg.DlgBalgPhysikalischeDaten;
import animation.pendel.DlgPendelMathematisch;
import animation.pendel.DlgPendelMechanisch;





public class MainFrame extends Component {

	private static final long serialVersionUID = 1L;
	private static final String TITLE = "GR Tool V1.2"; 

	private JMenuItem fileSave;
	private JMenuItem fileSaveAs;
	
	private JMenu toolsmenu;
	private JMenu animationMenus;
	private JMenu pendelMenus;
	private JMenu balgMenus;
	private JMenu spiralradMenus;
	
	private File current_file;
	
	private JFrame parent_frame;
	private MainFrame main_frame;
	private ImageBasePanel imageBasePanel;

	public JTextField tfCoord;

	BufferedImage img;

	public MainFrame(JFrame frame) {
		parent_frame = frame;
		main_frame = this;
		Properties prop = new Properties();
		try {
			prop.load(new FileInputStream("config.properties"));
			String filename = prop.getProperty("Filename");
			current_file = new File(filename);
		} catch (IOException ex) {
			current_file = null;
		}

	}

	public String getTitle() {
		return TITLE;
	}
	
	
	private void setAnimationMenus(JMenuBar menubar) {
		animationMenus = new JMenu("Animationen");
		menubar.add(animationMenus);
		pendelMenus = new JMenu("Pendel");
		animationMenus.add(pendelMenus);
		balgMenus = new JMenu("Balg");
		animationMenus.add(balgMenus);
		spiralradMenus = new JMenu("Spiralrad");
		animationMenus.add(spiralradMenus);
		
		JMenuItem mi_animateMathematischesPendel = new JMenuItem("Mathematisches Pendel");
		mi_animateMathematischesPendel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				DlgPendelMathematisch dlg = new DlgPendelMathematisch(main_frame);
				dlg.setVisible(true);
			}
		});
		pendelMenus.add(mi_animateMathematischesPendel);
		
		
		JMenuItem mi_animateMechanischesPendel = new JMenuItem("Mechanisches Pendel");
		mi_animateMechanischesPendel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				DlgPendelMechanisch dlg = new DlgPendelMechanisch(main_frame);
				dlg.setVisible(true);
			}
		});
		pendelMenus.add(mi_animateMechanischesPendel);
		
		JMenuItem mi_animateBalg = new JMenuItem("Physikalische Daten");
		mi_animateBalg.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				DlgBalgPhysikalischeDaten dlg = new DlgBalgPhysikalischeDaten(main_frame);
				dlg.setVisible(true);
			}
		});
		balgMenus.add(mi_animateBalg);
		
		JMenuItem mi_animateSpiralrad = new JMenuItem("Physikalische Daten");
		mi_animateSpiralrad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				DlgSpiralradPhysikalischeDaten dlg = new DlgSpiralradPhysikalischeDaten(main_frame);
				dlg.setVisible(true);
			}
		});
		spiralradMenus.add(mi_animateSpiralrad);
	}
	
	private void setToolsItems(JMenuBar menubar) {
		toolsmenu = new JMenu("Tools");
		menubar.add(toolsmenu);
		JMenuItem centerOfArea = new JMenuItem("Center of Area");
		centerOfArea.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mnu_centerOfArea();
			}
		});
		toolsmenu.add(centerOfArea);
		toolsmenu.setEnabled(false);
	}
	
	private void setFileMenuItems(JMenuBar menubar) {
		JMenu filemenu = new JMenu("File");
		menubar.add(filemenu);
		JMenuItem fileOpen = new JMenuItem("Open");
		fileOpen.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mnu_fileOpen();
			}
		});
		filemenu.add(fileOpen);

		filemenu.add(new JSeparator());
		
		fileSave = new JMenuItem("Save");
		fileSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mnu_fileSave();
			}
		});
		fileSave.setEnabled(false);
		filemenu.add(fileSave);
		
		fileSaveAs = new JMenuItem("Save as ...");
		fileSaveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mnu_fileSaveAs();
			}
		});
		fileSaveAs.setEnabled(false);
		filemenu.add(fileSaveAs);

		filemenu.add(new JSeparator());

		JMenuItem fileExit = new JMenuItem("Exit");
		fileExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				mnuExit();
			}
		});
		filemenu.add(fileExit);
		
	}
	
	public JMenuBar createMenuBar() {
		JMenuBar menubar = new JMenuBar();
		setFileMenuItems(menubar);
		setToolsItems(menubar);
		setAnimationMenus(menubar);
		return menubar;
	}

	public Container createContentPane() {

		imageBasePanel = new ImageBasePanel();
		imageBasePanel.setLayout(null);

		return imageBasePanel;
	}

	public ImageBasePanel getImageBasePanel() {
		return imageBasePanel;
	}
	
	
	public void mnuExit() {
		Properties prop = new Properties();
		if (current_file != null) {
			try {
				prop.setProperty("Filename", current_file.getPath());
				prop.store(new FileOutputStream("config.properties"), null);
	
			} catch (IOException ex) {
			}
		}
		System.exit(0);
	}

	public void mnu_fileOpen() {

		JFileChooser fc = new JFileChooser();
		fc.setSelectedFile(current_file);
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			current_file = fc.getSelectedFile();
			try {
				img = ImageIO.read(current_file);
				if (img == null) {
					JOptionPane.showMessageDialog(this,
							"Not a valid Image Format.", "Format Error",
							JOptionPane.ERROR_MESSAGE);
				} else {
					imageBasePanel.setImage(img);
					fileSave.setEnabled(true);
					fileSaveAs.setEnabled(true);
					toolsmenu.setEnabled(true);
					parent_frame.setTitle(getTitle()+" "+current_file.getName());
				}

			} catch (IOException e) {
				
				JOptionPane.showMessageDialog(this, ""
						+ "File cannot be opened", "File Error",
						JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public void mnu_fileSave() {

		BufferedImage img = imageBasePanel.captureActualImage();
		if (img == null) {
			JOptionPane.showMessageDialog(this, ""
					+ "Image cannot be captured", "Capturing Error",
					JOptionPane.ERROR_MESSAGE);
			return;
		}

		try {
			String extension = FileFilterUtils.getExtension(current_file);
			ImageIO.write(img, extension, current_file);
		} catch (IOException ex) {
			JOptionPane.showMessageDialog(this, ""
				+ "File cannot be saved", "File Error",
				JOptionPane.ERROR_MESSAGE);
		}
	}
	


	public void mnu_fileSaveAs() {

		BufferedImage img = imageBasePanel.captureActualImage();
		if (img == null) {
			JOptionPane.showMessageDialog(this, ""
					+ "Image cannot be captured", "Capturing Error",
					JOptionPane.ERROR_MESSAGE);
			return;
		}

		JFileChooser chooser = new JFileChooser();
		chooser.setSelectedFile(current_file);
		ImageFilter imageFilter = new ImageFilter();
		chooser.addChoosableFileFilter(imageFilter);
		chooser.setAcceptAllFileFilterUsed(false);

		int rval = chooser.showSaveDialog(this);
		if (rval == JFileChooser.APPROVE_OPTION) {
			File fileToSave = chooser.getSelectedFile();
			if (imageFilter.accept(fileToSave)) {
				if (fileToSave.exists()) {
					int response = JOptionPane.showConfirmDialog(null,
							"Overwrite existing file?", "Confirm Overwrite",
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (response == JOptionPane.CANCEL_OPTION)
						return;
				}
				
				try {
					String extension = FileFilterUtils.getExtension(fileToSave);
					// Write the filtered image in the selected format
					ImageIO.write(img, extension, fileToSave);
					current_file = fileToSave;
				} catch (IOException ex) {
					JOptionPane.showMessageDialog(this, ""
							+ "File cannot be saved", "File Error",
							JOptionPane.ERROR_MESSAGE);
				}
			} else {
				JOptionPane.showMessageDialog(this, ""
						+ "File cannot be saved (wrong extension).",
						"File Error", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private void mnu_centerOfArea() {
		
		imageBasePanel.showCenterOfAreaPanel();
	}
	
	public Dimension getPreferredSize() {
		if (img == null) {
			return new Dimension(guiConsts.mainFrmWidth,
					guiConsts.mainFrmHeight);
		} else {
			return new Dimension(img.getWidth(null), img.getHeight(null));
		}
	}

	private static void createAndShowGUI() {
		// Create and set up the window.
		JFrame frame = new JFrame();	
		frame.setResizable(false);
	
		final MainFrame application = new MainFrame(frame);
		frame.setTitle(application.getTitle());
		frame.setJMenuBar(application.createMenuBar());
		frame.setContentPane(application.createContentPane());
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowAdapter() {
			  public void windowClosing(WindowEvent e) {
			    application.mnuExit();
			  }
			});
		frame.setSize(guiConsts.mainFrmWidth, guiConsts.mainFrmHeight);
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		Toolkit.getDefaultToolkit().getSystemEventQueue().push(
				new TCPopupEventQueue());
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}

}
