package gui;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

public class RotatePanel extends JPanel {
	
	private static final long serialVersionUID = 1L;
	private ImageDrawPanel imageDrawPanel;
	@SuppressWarnings("unused")
	private ImageBasePanel imageBasePanel;
	
	JTextField tfCenter;
	JButton butRotate;
	
	double angle = 0.0;
	
	public RotatePanel(ImageBasePanel imageBasePanel) {
		setLayout(null);
		
		final int x0 = 10;
		
		final int h0 = 30;
		final int w1 = 80;
		final int x2 = 100;
		
		final int h = 20;
		final int h1 = 25;
		
		int yp = 0;
		this.imageBasePanel = imageBasePanel;
		this.imageDrawPanel = imageBasePanel.getImageDrawPanel();
		
		this.setSize(new Dimension(280,125));
		
		TitledBorder blackline = BorderFactory.createTitledBorder("Rotate");
		setBorder(blackline); 
		
		JLabel lbCenter = new JLabel("Center");
		lbCenter.setBounds(x0,h0,50,h);
		add(lbCenter);
		
		yp++;
		tfCenter = new JTextField();
		tfCenter.setBounds(x0,h0+yp*h1,w1,h);
		add(tfCenter);
		
		yp++;
		butRotate = new JButton("Rotate");
		butRotate.setBounds(x2,h0+yp*h1,2*w1+10,25);
		butRotate.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				cmdRotate();
			}});
		add(butRotate);
	}
	
	private boolean isNumeric(char ch) {
		switch (ch) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			return true;
		default:
			return false;
		}
	}
	
	private Coordinate checkCoordinate(String text,int xmax, int ymax) {
		Coordinate coord = new Coordinate();
		
		int i=0;
		int len = text.length();
		String xStr="";
		String yStr="";
		int x;
		int y;
		while (i<len) {
			if (isNumeric(text.charAt(i))) 
				break;
			else {
				i++;
			}
		}
		while (i<len) {
			if (isNumeric(text.charAt(i))==false) 
				break;
			else {
				xStr+=text.charAt(i);
				i++;
			}
		}
		while (i<len) {
			if (isNumeric(text.charAt(i))) 
				break;
			else {
				i++;
			}
		}
		while (i<len) {
			if (isNumeric(text.charAt(i))==false) 
				break;
			else {
				yStr+=text.charAt(i);
				i++;
			}
		}
		if ((xStr.length()==0)||(yStr.length()==0)) {
			coord.setValid(false);
		}
		else {
			x = Integer.valueOf(xStr);
			y = Integer.valueOf(yStr);
			if ((x<xmax)&&(y<ymax)) {
				coord.setValid(true);
				coord.setOutOfRange(false);
				coord.setX(x);
				coord.setY(y);
			}
			else {
				coord.setValid(false);
				coord.setOutOfRange(true);
				coord.setX(x);
				coord.setY(y);
			}
		}
		return coord;
	}
	
	private BufferedImage rotate(BufferedImage inputImage, double angle, int cx, int cy) { 
		  int width;
		  int height;
	    
	    
	    width = inputImage.getWidth();
	    height = inputImage.getHeight();
	    
	 
	    BufferedImage rotatedBi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	 
	    AffineTransform at = new AffineTransform();
	    at.rotate(Math.toRadians( angle ), cx, cy );
	 
	  
	    Graphics2D g = (Graphics2D) rotatedBi.getGraphics();
	    g.fillRect(0,0,width,height);
	    g.setTransform(at);
	    g.drawImage(inputImage, 0, 0, null);
	 
	 
	    return rotatedBi;
	}
	
	private void cmdRotate() {
		Coordinate coord = checkCoordinate(tfCenter.getText(),
				guiConsts.imageDrawPanelWidth,
				guiConsts.imageDrawPanelHeight);
		
		if (coord.isValid()==false) {
			if (coord.isOutOfRange()) {
				JOptionPane.showMessageDialog(this, ""
						+ "Center value is out of range.", "Range Error",
						JOptionPane.ERROR_MESSAGE);
			}
			else {
				JOptionPane.showMessageDialog(this, ""
						+ "Center value"+tfCenter.getText()+" is not a valid.\n\nExamples for valid entries:\nX:122 Y:98\nx: 122, y: 98\n122 98", "Number Format Error",
						JOptionPane.ERROR_MESSAGE);
			}
			return;
		}
		BufferedImage image = imageDrawPanel.getClonedImage();
		if (image == null) return;
		imageDrawPanel.setRotatedImage(rotate(image, angle, coord.getX(),coord.getY()));
		angle +=10;
		imageDrawPanel.repaint();
		
		
	}
	
	private class Coordinate {
		int x;
		int y;
		boolean valid;
		boolean outOfRange;
		public int getX() {
			return x;
		}
		public void setX(int x) {
			this.x = x;
		}
		public int getY() {
			return y;
		}
		public void setY(int y) {
			this.y = y;
		}
		public boolean isValid() {
			return valid;
		}
		public void setValid(boolean valid) {
			this.valid = valid;
		}
		public boolean isOutOfRange() {
			return outOfRange;
		}
		public void setOutOfRange(boolean outOfRange) {
			this.outOfRange = outOfRange;
		}
		
	}

}
