package printing;

import static java.lang.Math.cos;
import static java.lang.Math.sin;
import static printing.PrintUtils.TEXTKOERPER_BREITE;
import static printing.PrintUtils.RAND_OBEN;
import static printing.PrintUtils.RAND_UNTEN;

import static printing.PrintUtils.fntNormal;
import static printing.PrintUtils.fntTitleBold;
import static printing.PrintUtils.fntData;
import static printing.PrintUtils.iResMul;
import static printing.PrintUtils.pnth;
import static printing.PrintUtils.pntw;
import static printing.PrintUtils.pntx;
import static printing.PrintUtils.pnty;
import static printing.PrintUtils.putValue;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.Book;
import java.awt.print.PageFormat;
import java.awt.print.Paper;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.text.NumberFormat;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import animation.spiralrad.SpiralradDaten;

public class DruckAuftragSpirale {

	private NumberFormat f0 = NumberFormat.getInstance();
	private NumberFormat f2 = NumberFormat.getInstance();

	private int seite = 0;
	private JFrame frame;

	private SpiralradDaten spiralradDaten;

	public DruckAuftragSpirale(JFrame frame, SpiralradDaten spiralradDaten) {
		this.frame = frame;
		this.spiralradDaten = spiralradDaten;
		PrintUtils.init();
		if (this.frame != null) {
			this.frame
					.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		}

		int anzahlSeiten;

		// Create a print job
		PrinterJob pj = PrinterJob.getPrinterJob();
		pj.setJobName("Spirale");

		try {

			if (pj.printDialog()) {

				Paper paper = new Paper();
				paper.setSize(595, 841);
				// paper.setImageableArea(56,14,567,827);
				paper.setImageableArea(38, 14, 567, 827);
				PageFormat pageFormat = new PageFormat();
				pageFormat.setPaper(paper);
				pageFormat.setOrientation(PageFormat.PORTRAIT);

				Book book = new Book();
				pj.setPageable(book);

				f0.setMinimumFractionDigits(0);
				f0.setMaximumFractionDigits(0);

				f2.setMinimumFractionDigits(0);
				f2.setMaximumFractionDigits(2);

				seite = 0;
				anzahlSeiten = 1;
				book.append(new SpiralenBlatt(seite, anzahlSeiten), pageFormat);

				pj.print();
			}
		} catch (PrinterException pe) {
			System.err.println("DruckAuftrag nicht druckbar. " + (seite + 1));
			JOptionPane.showMessageDialog(null,
					"Druckauftrag nicht druckbar (Seite " + (seite) + ").",
					"FEHLER", JOptionPane.ERROR_MESSAGE);
		}
		if (this.frame != null) {
			this.frame.setCursor(Cursor
					.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	private class SpiralenBlatt extends Component implements Printable {

		private static final long serialVersionUID = 1L;

		final double skalierung = 0.35;
		private int seite;

		@SuppressWarnings("unused")
		private int anzahlSeiten;

		private int innenRadius;
		private int aussenRadius;
		private int anzahlSpiralen;
		private double umdrehungen;


		public SpiralenBlatt(int seite, int anzahlSeiten) {
			this.seite = seite;
			this.anzahlSeiten = anzahlSeiten;
			this.innenRadius = spiralradDaten.getInnenRadius();
			this.aussenRadius = spiralradDaten.getAussenRadius();
			this.anzahlSpiralen = spiralradDaten.getAnzahlSpiralen();
			this.umdrehungen = spiralradDaten.getTeilung();
		}

		private void zeichneSpirale(Graphics2D g2, 
				int center_x, 
				int center_y, 
				double phi) {

			double alpha;
			double spirale_x_neu;
			double spirale_y_neu;
			
			g2.setColor(Color.black);

			double radius = innenRadius;
			int steps = (int) Math.round(360.0 * umdrehungen);
			double deltaRadius = ((double) (aussenRadius - innenRadius)) / steps;

			double spirale_x_alt = center_x - cos(phi) * radius * skalierung;
			double spirale_y_alt = center_y - sin(phi) * radius * skalierung;

			g2.setColor(Color.black);
			for (int n = 0; n < steps; n++) {
				alpha = Math.toRadians(n) + phi;
				spirale_x_neu = center_x - cos(alpha) * radius * skalierung;
				spirale_y_neu = center_y - sin(alpha) * radius * skalierung;

				g2.drawLine(pntx(spirale_x_alt), 
						pnty(spirale_y_alt),
						pntx(spirale_x_neu), 
						pnty(spirale_y_neu));

				spirale_x_alt = spirale_x_neu;
				spirale_y_alt = spirale_y_neu;
				radius = radius + deltaRadius;
			}
		}

		public int print(Graphics g, PageFormat pf, int iPage)
				throws PrinterException {
			final int FONTSIZE = 12;

			double yText;
			double ylow = 262;

			if (seite != iPage)
				return NO_SUCH_PAGE;

			try {

				Graphics2D g2 = (Graphics2D) g;
				g2.translate(pf.getImageableX(), pf.getImageableY());
				g2.scale(1.0 / iResMul, 1.0 / iResMul);
				g2
						.setFont(new Font("SansSerif", Font.PLAIN, FONTSIZE
								* iResMul));
				g2.setColor(Color.black);

				// Titel zeichnen
				yText = 20.;
				g2.setFont(fntTitleBold);
				if (anzahlSpiralen == 1) {
					g2.drawString("Spiralen-Rad", pntx(3), pnty(yText));
				} else {
					g2.drawString("Multispiralen-Rad", pntx(3), pnty(yText));
				}
				g2.setFont(fntNormal);
				yText += 6;
				g2.drawString("Anzahl Spiralen:", pntx(3), pnty(yText));
				g2.drawString(putValue(0, anzahlSpiralen), pntx(40),
						pnty(yText));
				yText += 4;
				g2.drawString("Anzahl Umdrehungen:", pntx(3), pnty(yText));
				g2.drawString(putValue(1, umdrehungen), pntx(40), pnty(yText));
				yText += 4;
				g2.drawString("Innenradius [Pixel]:", pntx(3), pnty(yText));
				g2.drawString(putValue(0, innenRadius), pntx(40), pnty(yText));
				yText += 4;
				g2.drawString("Aussenradius [Pixel]:", pntx(3), pnty(yText));
				g2.drawString(putValue(0, aussenRadius), pntx(40), pnty(yText));

				yText += 4;

				g2.fillRect(pntx(0), pnty(yText), pntw(180), 2);
				yText += 7.5;

				// Blattrand zeichnen
				g2.fillRect(pntx(0), pnty(ylow), pntw(TEXTKOERPER_BREITE), 2);
				g2.fillRect(pntx(0), pnty(RAND_OBEN), pntw(TEXTKOERPER_BREITE),
						2);
				g2.fillRect(pntx(0), pnty(RAND_OBEN), 2,
						pnth(ylow - RAND_UNTEN));
				g2.fillRect(pntx(TEXTKOERPER_BREITE), pnty(RAND_OBEN), 2,
						pnth(ylow - RAND_UNTEN));

				// Rad zeichnen

				int cx = 90;
				int cy = 135;
				g2.drawArc(pntx(cx) - pntw(skalierung * aussenRadius), pnty(cy)
						- pnth(skalierung * aussenRadius), pntw(skalierung
						* aussenRadius * 2.0), pnth(skalierung * aussenRadius
						* 2.0), 0, 360);
				g2.drawLine(pntx(cx - 5), pnty(cy), pntx(cx + 5), pnty(cy));
				g2.drawLine(pntx(cx), pnty(cy - 5), pntx(cx), pnty(cy + 5));

				double deltaPhi = 1.0 / (double) anzahlSpiralen;
				for (int i = 0; i < anzahlSpiralen; i++) {
					zeichneSpirale(g2, cx, cy, 2.0 * Math.PI * (double) i
							* deltaPhi);
				}
				g2.setFont(fntData);
				g2.drawString("1 Pixel = 0.35mm", pntx(3), pnty(250));
			} catch (Exception ex) {
				throw new PrinterException(ex.getMessage());
			}

			return PAGE_EXISTS;
		}
	}

}
