package printing;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.util.Properties;

import javax.swing.JFrame;



public final class PrintUtils extends Component {
	

	private static final long serialVersionUID = 1L;

	private static DecimalFormat df = new DecimalFormat(); 
	
	public static int iResMul = 4;  // 1 = 72 dpi; 4 = 288 dpi
	public static double PNT_MM = (iResMul * 72.)/25.4;
	
	// Nurmales Fontset fuer die Ausgaben
	public static Font fntTitle = new Font("Helvetica-Bold", Font.PLAIN, 12*iResMul);
	public static Font fntTitleBold = new Font("Helvetica-Bold", Font.BOLD, 12*iResMul);
	public static Font fntNormal= new Font("Helvetica", Font.PLAIN, 10*iResMul);
	public static Font fntNormalBold= new Font("Helvetica-Bold", Font.BOLD, 10*iResMul);
	public static Font fnt9= new Font("Helvetica", Font.PLAIN, 9*iResMul);
	public static Font fnt9Bold= new Font("Helvetica-Bold", Font.BOLD, 9*iResMul);
	
	// Fontset fuer die Diagramme
	public static Font fntBez =  new Font("Helvetica", Font.PLAIN, 10*iResMul);
	public static Font fntBezBold =  new Font("Helvetica", Font.BOLD, 10*iResMul);
	public static Font fntData =  new Font("Helvetica", Font.PLAIN, 8*iResMul);
	public static Font fntDataBold =  new Font("Helvetica", Font.BOLD, 8*iResMul);
	
	
	
  // Da alle Grafikausgaben eber pntx(..), pnty(..) skaliert werden,
	// kann mit iPosX und iPosY ein Offset zum linken bzw. oberen Beginn
	// des Textkoerpers angegeben werden.
	public static int iPosX=6; // in mm (Rand frei lassen fuer den senkrecht geschriebenen Verteiler links)
	public static int iPosY=1; // in mm
	

	
	public static final int SEITENHOEHE=297; // in mm (A4)
	public static final int SEITENBREITE=210; // in mm (A4)
	
	public static final int RAND_OBEN = 15; // in mm, 
	public static final int RAND_UNTEN = 15; // in mm, 
	public static final int TEXTKOERPER_BREITE = 180; // in mm
	public static final int TEXTKOERPER_HOEHE = 262; // in mm

	public static final int ANZAHL_SEITEN_BERECHNEN = 0;
	public static final int DRUCKEN = 1;
	
	private static DecimalFormat df0 = new DecimalFormat();
	
	
	public  PrintUtils (JFrame frame) {
		df0.setMaximumFractionDigits(0);
		df0.setMinimumFractionDigits(0);
		df0.setGroupingUsed(false);
		df0.setDecimalSeparatorAlwaysShown(false);
		
		df.setMinimumFractionDigits(0); 
		df.setGroupingUsed(false);
		df.setDecimalSeparatorAlwaysShown(false); 
	}
	
	public static void init() {
		df0.setMaximumFractionDigits(0);
		df0.setMinimumFractionDigits(0);
		df0.setGroupingUsed(false);
		df0.setDecimalSeparatorAlwaysShown(false);
		
		df.setMinimumFractionDigits(0); 
		df.setGroupingUsed(false);
		df.setDecimalSeparatorAlwaysShown(false); 
		
	}

	public static String Str2Numstr(String strnum) {
		double dblnum;

  	try { 
  		dblnum=Double.valueOf(strnum);
			return df0.format(dblnum); 
		} 
		catch (Exception e) { 
			return strnum;// Falls Zahlenformat falsch, Inhalt direkt uebernehmen
		}
	}
	

	public static int pntx(double v) {
		return iPosX*(int)PNT_MM+(int)(v*PNT_MM);
	}
	
	public static int pnty(double v) {
		return iPosY+(int)(v*PNT_MM);
	}
	
	public static int pntw(double v) {
		return (int)(v*PNT_MM);
	}
	
	public static int pnth(double v) {
		return (int)(v*PNT_MM);
	}
	
	public static double getStringWidth(Graphics2D g2,String text) { // in mm
		FontRenderContext frc;
		frc = g2.getFontRenderContext(); // Aktuelle Aufloesung in 1/288 inch
		Rectangle2D bounds = g2.getFont().getStringBounds(text, frc); 
		return ((25.4*bounds.getWidth())/(iResMul*72)); // in mm
	}
	public static double getStringWidth(Graphics2D g2,Font font, String text) { // in mm
		Font oldFont=g2.getFont();
		g2.setFont(font);
		FontRenderContext frc;
		frc = g2.getFontRenderContext(); // Aktuelle Aufloesung in 1/288 inch
		Rectangle2D bounds = g2.getFont().getStringBounds(text, frc); 
		g2.setFont(oldFont);
		return ((25.4*bounds.getWidth())/(iResMul*72)); // in mm
	}
	
	public static double getAscent(Graphics2D g2) {
		// Hhe eines normalen Grossbuchstabens in mm
		LineMetrics lm = g2.getFont().getLineMetrics("A",g2.getFontRenderContext());
		return 25.4*lm.getAscent()/(iResMul*72); // in mm
	}
	
	public static double getStringHeight(Graphics2D g2, String text) { // in mm
		// Gesamthhe eines Strings
		FontRenderContext frc;
		frc = g2.getFontRenderContext(); // Aktuelle Aufloesung in 1/288 inch
		Rectangle2D bounds = g2.getFont().getStringBounds(text, frc);
		return ((25.4*bounds.getHeight())/(iResMul*72)); // in mm
	}
	public static double getStringHeight(Graphics2D g2,Font font, String text) { // in mm
		Font oldFont=g2.getFont();
		g2.setFont(font);
		FontRenderContext frc;
		frc = g2.getFontRenderContext(); // Aktuelle Aufloesung in 1/288 inch
		Rectangle2D bounds = g2.getFont().getStringBounds(text, frc);
		g2.setFont(oldFont);
		return ((25.4*bounds.getHeight())/(iResMul*72)); // in mm
	}
	

	public static void drawRect(Graphics2D g2, double x, double y, double w, double h, double thickness) {
		Font oldFont = g2.getFont();
		g2.setColor(Color.BLACK);
		g2.fillRect(pntx(x),pnty(y),pntw(w),pnth(thickness));
		g2.fillRect(pntx(x),pnty(y+h),pntw(w),pnth(thickness));
		g2.fillRect(pntx(x),pnty(y),pntw(thickness),pnth(h));
		g2.fillRect(pntx(x+w),pnty(y),pntw(thickness),pnth(h+thickness));
		g2.setFont(oldFont);
	}
	
	public static double Feld(Graphics2D g2, double xlabel,double y, String label,double xdim, String dim,double xitem, String item) {
		// Beispiel:
		// Luftspalt pro Saeule      [mm] : 6    <- Dimension ist linksbuendig
		// Wirkwiderstand            [Ohm]: 80
		
		g2.setFont(fntNormal); 
		g2.drawString(label,	pntx(xlabel),		pnty(y));
		g2.drawString(dim,		pntx(xdim),			pnty(y));
		g2.drawString(":",    pntx(xitem-3.), 	pnty(y));
		g2.drawString(item,		pntx(xitem),  		pnty(y));
		return y+getStringHeight(g2,label+dim+":"+item); 
	}
	
	public static double Feld_dim_rb(Graphics2D g2, double xlabel,double y, String label,double xdim, String dim,double xitem, String item) {
		// Beispiel
		// Luftspalt pro Saeule       [mm]: 6    <- Dimension ist rechtsbuendig
		// Wirkwiderstand            [Ohm]: 80
		g2.setFont(fntNormal); 
		g2.drawString(label,	pntx(xlabel),		pnty(y));
		double width=getStringWidth(g2,dim);
		g2.drawString(dim,	pntx(xdim-width),		pnty(y));
		g2.drawString(":",    pntx(xitem-3.), 	pnty(y));
		g2.drawString(item,		pntx(xitem),  		pnty(y));
		return y+getStringHeight(g2,label+dim+":"+item); 
	}
	
	
	public static double Feld(Graphics2D g2, double xlabel,double y, String label,double xitem, String item) {
		g2.setFont(fntNormal); 
		g2.drawString(label,	pntx(xlabel),		pnty(y));
		g2.drawString(":",    pntx(xitem-3.), pnty(y));
		g2.drawString(item,		pntx(xitem),  	pnty(y));
		return y+getStringHeight(g2,label+":"+item); 
	}
	
	
	public static double Feld (Graphics2D g2, double x, double y, String text) {
		// Geeignet fuer: Kerngewicht (mit text="Kerngewicht")  
		g2.drawString(text,	pntx(x),		pnty(y));
		return y+getStringHeight(g2,text); 
	}
	public static double Feld (Graphics2D g2, double x, double y, String text,Font fnt,String pos) {
		// Geeignet fuer: Kerngewicht (mit text="Kerngewicht")
		double width;
		Font oldFont;
		oldFont=g2.getFont();
		g2.setFont(fnt);
		width=getStringWidth(g2,text);
		if (pos.compareTo("left")==0) {
			g2.drawString(text,	pntx(x),		pnty(y));
		}
		if (pos.compareTo("center")==0) {
			g2.drawString(text,	pntx(x-(width/2)),		pnty(y));
		}
		if (pos.compareTo("right")==0) {
			g2.drawString(text,	pntx(x-width),		pnty(y));
		}
		g2.setFont(oldFont);
		return y+getStringHeight(g2,text); 
	}
	public static double Feld (Graphics2D g2, double x, double y, String label,String dim,String wert) {
		// Geeignet fuer: Kerngewicht [kg]: 50 (mit label="Gewicht" dim="[kg]" item = "50")
		String s=label+" "+dim+": "+wert;
		g2.drawString(s,	pntx(x),		pnty(y));
		return y+getStringHeight(g2,label+dim+wert); 
	}
	public static double Feld (Graphics2D g2, double x, double y,String label,String wert) {
		// Geeignet fuer: Kunde: ELMAP (mit label="Kunde" wert="ELMAP)
		String s=label+": "+wert;
		g2.drawString(s,	pntx(x),		pnty(y));
		return y+getStringHeight(g2,label+wert); 
	}
	public static double Feld (Graphics2D g2, double x, double y, String label,String wert, Font fnt) {
		// Geeignet fuer: Kunde: ELMAP (bolded Font) (mit label="Kunde" wert="ELMAP) 
		double width;
		double height;
		Font oldFont =g2.getFont();
		String s=label+": ";
		g2.drawString(s,	pntx(x),		pnty(y));
		width=getStringWidth(g2,s);
		g2.setFont(fnt);
		g2.drawString(wert,	pntx(x+width),		pnty(y));
		height=getStringHeight(g2,label+wert); 
		g2.setFont(oldFont);
		return y+height;
		
	}
	public static double MultipleLines(Graphics2D g2, double x, double yT, String text, Font fnt) {
		double height;
		double y=yT;
		Font oldFont;
		String zeile="";
		
		oldFont=g2.getFont();
		g2.setFont(fnt);
		height=getStringHeight(g2,text);
    char[] text2 = new char[text.length()];
    try{
    	text.getChars(0,text.length(), text2,0);
    }catch(IndexOutOfBoundsException ioobe) {}
    for (int tmp = 0; tmp<text2.length; tmp++){
    	switch (text2[tmp]) {
    		case '\n':
    			g2.drawString(zeile,	pntx(x),pnty(y));
    			y+=height;
    			zeile="";
    			break;
    		case '\t':
    			for (int ttt=0;ttt<12; ttt++) {
    				zeile+=" ";
    			}
    			break;
    		default: 
    			zeile+=text2[tmp];
    			break;
    	} 
    }
    if (zeile.length()>0) {
    	g2.drawString(zeile,	pntx(x),pnty(y));
    	y+=height;
    }
		g2.setFont(oldFont);
		return y;
	
	}
	public static void PfeilRechts(Graphics2D g2, double x, double y, double l) {
		// x,y [mm}, Pfeillnge l [mm]
		
    // momentane Grafikeinstellungen sichern

		Stroke oldStroke = g2.getStroke();
  	final BasicStroke stroke = new BasicStroke(1.6f);
  	g2.setStroke(stroke);

  	GeneralPath path = new GeneralPath();			
		path.moveTo((float)pntx(x),(float)pnty(y));
		path.lineTo((float)pntx(x+l),(float)pnty(y));
		path.moveTo((float)pntx(x+l-1.5),(float)pnty(y-0.5));
		path.lineTo((float)pntx(x+l),(float)pnty(y));
		path.moveTo((float)pntx(x+l-1.5),(float)pnty(y+0.5));
		path.lineTo((float)pntx(x+l),(float)pnty(y));
		path.closePath();
  	g2.draw(path);
  	
		// urspruengliche Grafikeinstellungen zureckladen
  	g2.setStroke(oldStroke);
	}
	public static double Rahmen(Graphics2D g2, Font fnt,double h, double y,int l, int o, int r, int u, String pos,double x0, double x1,String text) {
		// Rahmen(g2,7.5,yText,0,0,1,0,"center", 0,		3.5,		"Nr");
		double width;
		double ytext;
		double centerX;
		double thickness = 0;
		
		Rahmen(g2,h,y,l,o,r,u,x0,x1); // zeichnet einen leeren Rahmen
		if ((o==2)||(u==2)) thickness=0.4;
		if ((o==3)||(u==3)) thickness=0.8;
		g2.setFont(fnt);
		width=getStringWidth(g2,text); // in mm
		ytext = y+(h-thickness)/2+getStringHeight(g2,text)/3;  //+getStringHeight(g2,text)*1.2-thickness)/2;
		if (pos.compareTo("center")==0){
			centerX=(x1-x0)/2; // in mm
			g2.drawString(text, pntx(x0+centerX-width/2), pnty(ytext));
		}
		if (pos.compareTo("left")==0){
			g2.drawString(text, pntx(x0+1), pnty(ytext));
		}

		if (pos.compareTo("right")==0){
			g2.drawString(text, pntx(x1-width-1), pnty(ytext));
		}
		return y+h; // unterste benuetze y-Koordinate
	}
	
	public static double Rahmen(Graphics2D g2, Font fnt,double h, double y,int l, int o, int r, int u,double x0, double x1,String beschriftung, String einheit) {
		// Rahmen mit Beschriftung linksbuendig und Einheit rechtsbuendig 
		double width;
		double ytext;
		double thickness = 0;
		
		Rahmen(g2,h,y,l,o,r,u,x0,x1); // zeichnet einen leeren Rahmen
		if ((o==2)||(u==2)) thickness=0.4;
		if ((o==3)||(u==3)) thickness=0.8;
		g2.setFont(fnt);
		
		ytext = y+(h-thickness)/2+getStringHeight(g2,beschriftung)/3;  //+getStringHeight(g2,text)*1.2-thickness)/2;
		
		// Beschriftung linksbuendig 1 mm vom linken Rand enfernt zeichnen
		width=getStringWidth(g2,beschriftung); // in mm
		g2.drawString(beschriftung, pntx(x0+1), pnty(ytext));
	
		// Einheit rechtsbuendig 1 mm vom rechten Rand enfernt zeichnen
		width=getStringWidth(g2,einheit); // in mm
		g2.drawString(einheit, pntx(x1-width-1), pnty(ytext));
	
		return y+h; // unterste benuetze y-Koordinate
	}
	
	

	

	public static double Rahmen(Graphics2D g2, double h, double y,int l, int o, int r, int u, double x0, double x1) {
  	Stroke oldStroke = g2.getStroke();
  	Object oldRenderingHint_AntiAliasing = g2.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
  	Object oldRenderingHint_Rendering = g2.getRenderingHint(RenderingHints.KEY_RENDERING);
 
  	
  	if (l>0) { // Rand links
  		g2.setStroke(new BasicStroke(l*1.8f));
  		g2.drawLine(pntx(x0), pnty(y),	pntx(x0),pnty(y+h));
  	}
  	if (o>0) { // Rand oben
  		g2.setStroke(new BasicStroke(o*1.8f));
  		g2.drawLine(pntx(x0), pnty(y),	pntx(x1),pnty(y));
  	}
  	if (r>0) { // Rand rechts
  		g2.setStroke(new BasicStroke(r*1.8f));
  		g2.drawLine(pntx(x1), pnty(y),	pntx(x1),pnty(y+h));
  	}
   	if (u>0) { // Rand unten
  		g2.setStroke(new BasicStroke(u*1.8f));
  		g2.drawLine(pntx(x0), pnty(y+h),pntx(x1),pnty(y+h)); 
  	}
 
   	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,oldRenderingHint_AntiAliasing);
   	g2.setRenderingHint(RenderingHints.KEY_RENDERING,oldRenderingHint_Rendering);
  	g2.setStroke(oldStroke);
  	
		return y+h; // unterste benuetze y-Koordinate
	}
	
	public static String putValue(int MaximumFractionDigits, String strvalue) {
		df.setMaximumFractionDigits(MaximumFractionDigits); 
		return df.format(Double.valueOf(strvalue));
	}
	
	public static String putValue(int MaximumFractionDigits, double value) {
		df.setMaximumFractionDigits(MaximumFractionDigits); 
		return df.format(value);
	}

	public static String putValue(int MaximumFractionDigits, String key, Properties p) {
		df.setMaximumFractionDigits(MaximumFractionDigits); 
		return df.format(Double.valueOf(p.getProperty(key)));
	}
	
	public static void Tabelle(Graphics2D g2,
			double y0,
			String titel,
			String label[][],
			String item[][],
			
			int dim[],
			double lx[],
			double ix[]) {

		double yText;
		double y1;
		double h;

		yText=y0;
		g2.setFont(fntNormalBold);
		g2.drawString(titel, 		pntx(5),pnty(yText));
		yText+=2.;
		y1=yText;
		g2.fillRect(pntx(3),pnty(yText),pntw(174),2);
		h=0;
		for (int k=0;k<dim[1];k++) {
			yText+=5.5;
			h+=5.5;
			g2.setFont(fntNormal);
			for (int n=0;n<dim[0];n++) {
				g2.drawString(label[k][n],pntx(lx[n]),pnty(yText));
				g2.drawString(item[k][n],	pntx(ix[n]),pnty(yText));
			}
			yText+=2.;
			h+=2.;
			g2.fillRect(pntx(3),pnty(yText),pntw(174),2);
		}


		g2.fillRect(pntx(3),pnty(y1),2,pnth(h));
		for (int n=0;n<dim[0];n++) {
			if (n>0) g2.fillRect(pntx(lx[n]-3),pnty(y1),2,pnth(h));
			g2.fillRect(pntx(ix[n]-3),pnty(y1),2,pnth(h));
		}
		g2.fillRect(pntx(177),pnty(y1),2,pnth(h));

	}

	

}
